@@ -15,18 +15,17 @@
 import itertools
 import os.path
 
-from oslo_config import cfg
 from oslo_log import log
-import six
 
-from keystone.catalog import core
+from keystone.catalog.backends import base
+from keystone.common import utils
+import keystone.conf
 from keystone import exception
-from keystone.i18n import _LC
 
 
 LOG = log.getLogger(__name__)
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 def parse_templates(template_lines):
@@ -56,7 +55,7 @@ def parse_templates(template_lines):
     return o
 
 
-class Catalog(core.Driver):
+class Catalog(base.CatalogDriverBase):
     """A backend that generates endpoints for the Catalog based on templates.
 
     It is usually configured via config entries that look like:
@@ -65,13 +64,12 @@ class Catalog(core.Driver):
 
     and is stored in a similar looking hierarchy. Where a value can contain
     values to be interpolated by standard python string interpolation that look
-    like (the % is replaced by a $ due to paste attempting to interpolate on
-    its own:
+    like (the % is replaced by a $):
 
       http://localhost:$(public_port)s/
 
     When expanding the template it will pass in a dict made up of the conf
-    instance plus a few additional key-values, notably tenant_id and user_id.
+    instance plus a few additional key-values, notably project_id and user_id.
 
     It does not care what the keys and values are but it is worth noting that
     keystone_compat will expect certain keys to be there so that it can munge
@@ -103,7 +101,7 @@ class Catalog(core.Driver):
             with open(template_file) as f:
                 self.templates = parse_templates(f)
         except IOError:
-            LOG.critical(_LC('Unable to open template file %s'), template_file)
+            LOG.critical('Unable to open template file %s', template_file)
             raise
 
     # region crud
@@ -132,8 +130,8 @@ class Catalog(core.Driver):
         raise exception.NotImplemented()
 
     def _list_services(self, hints):
-        for region_ref in six.itervalues(self.templates):
-            for service_type, service_ref in six.iteritems(region_ref):
+        for region_ref in self.templates.values():
+            for service_type, service_ref in region_ref.items():
                 yield {
                     'id': service_type,
                     'enabled': True,
@@ -163,8 +161,8 @@ class Catalog(core.Driver):
         raise exception.NotImplemented()
 
     def _list_endpoints(self):
-        for region_id, region_ref in six.iteritems(self.templates):
-            for service_type, service_ref in six.iteritems(region_ref):
+        for region_id, region_ref in self.templates.items():
+            for service_type, service_ref in region_ref.items():
                 for key in service_ref:
                     if key.endswith('URL'):
                         interface = key[:-3]
@@ -195,14 +193,14 @@ class Catalog(core.Driver):
     def delete_endpoint(self, endpoint_id):
         raise exception.NotImplemented()
 
-    def get_catalog(self, user_id, tenant_id):
+    def get_catalog(self, user_id, project_id):
         """Retrieve and format the V2 service catalog.
 
         :param user_id: The id of the user who has been authenticated for
             creating service catalog.
-        :param tenant_id: The id of the project. 'tenant_id' will be None in
+        :param project_id: The id of the project. 'project_id' will be None in
             the case this being called to create a catalog to go in a domain
-            scoped token. In this case, any endpoint that requires a tenant_id
+            scoped token. In this case, any endpoint that requires a project_id
             as part of their URL will be skipped.
 
         :returns: A nested dict representing the service catalog or an
@@ -213,10 +211,10 @@ class Catalog(core.Driver):
             itertools.chain(CONF.items(), CONF.eventlet_server.items()))
         substitutions.update({'user_id': user_id})
         silent_keyerror_failures = []
-        if tenant_id:
+        if project_id:
             substitutions.update({
-                'tenant_id': tenant_id,
-                'project_id': tenant_id,
+                'tenant_id': project_id,
+                'project_id': project_id,
             })
         else:
             silent_keyerror_failures = ['tenant_id', 'project_id', ]
@@ -231,17 +229,64 @@ class Catalog(core.Driver):
                 service_data = {}
                 try:
                     for k, v in service_ref.items():
-                        formatted_value = core.format_url(
+                        formatted_value = utils.format_url(
                             v, substitutions,
                             silent_keyerror_failures=silent_keyerror_failures)
                         if formatted_value:
                             service_data[k] = formatted_value
-                except exception.MalformedEndpoint:
+                except exception.MalformedEndpoint:  # nosec(tkelsey)
                     continue  # this failure is already logged in format_url()
                 catalog[region][service] = service_data
 
         return catalog
 
+    def get_v3_catalog(self, user_id, project_id):
+        """Retrieve and format the current V3 service catalog.
+
+        This implementation builds the V3 catalog from the V2 catalog.
+
+        :param user_id: The id of the user who has been authenticated for
+            creating service catalog.
+        :param project_id: The id of the project. 'project_id' will be None in
+            the case this being called to create a catalog to go in a domain
+            scoped token. In this case, any endpoint that requires a project_id
+            as part of their URL will be skipped.
+
+        :returns: A list representing the service catalog or an empty list
+
+        """
+        v2_catalog = self.get_catalog(user_id, project_id)
+        v3_catalog = {}
+
+        for region_name, region in v2_catalog.items():
+            for service_type, service in region.items():
+                if service_type not in v3_catalog:
+                    v3_catalog[service_type] = {
+                        'type': service_type,
+                        'endpoints': []
+                    }
+
+                for attr, value in service.items():
+                    # Attributes that end in URL are interfaces. In the V2
+                    # catalog, these are internalURL, publicURL, and adminURL.
+                    # For example, <region_name>.publicURL=<URL> in the V2
+                    # catalog becomes the V3 interface for the service:
+                    # { 'interface': 'public', 'url': '<URL>', 'region':
+                    #   'region: '<region_name>' }
+                    if attr.endswith('URL'):
+                        v3_interface = attr[:-len('URL')]
+                        v3_catalog[service_type]['endpoints'].append({
+                            'interface': v3_interface,
+                            'region': region_name,
+                            'url': value,
+                        })
+                        continue
+
+                    # Other attributes are copied to the service.
+                    v3_catalog[service_type][attr] = value
+
+        return list(v3_catalog.values())
+
     def add_endpoint_to_project(self, endpoint_id, project_id):
         raise exception.NotImplemented()
 
