@@ -19,26 +19,27 @@ import os
 import sys
 import uuid
 
+import migrate
 from oslo_config import cfg
+from oslo_db.sqlalchemy import migration
 from oslo_log import log
-from oslo_log import versionutils
 from oslo_serialization import jsonutils
 import pbr.version
 
+from keystone.cmd import bootstrap
 from keystone.cmd import doctor
 from keystone.common import driver_hints
-from keystone.common import openssl
+from keystone.common import fernet_utils
 from keystone.common import sql
-from keystone.common.sql import migration_helpers
+from keystone.common.sql import upgrades
 from keystone.common import utils
 import keystone.conf
 from keystone.credential.providers import fernet as credential_fernet
 from keystone import exception
 from keystone.federation import idp
 from keystone.federation import utils as mapping_engine
-from keystone.i18n import _, _LE, _LI, _LW
+from keystone.i18n import _
 from keystone.server import backends
-from keystone import token
 
 
 CONF = keystone.conf.CONF
@@ -62,20 +63,7 @@ class BootStrap(BaseApp):
     name = "bootstrap"
 
     def __init__(self):
-        self.load_backends()
-        self.project_id = uuid.uuid4().hex
-        self.role_id = uuid.uuid4().hex
-        self.service_id = None
-        self.service_name = None
-        self.username = None
-        self.project_name = None
-        self.role_name = None
-        self.password = None
-        self.public_url = None
-        self.internal_url = None
-        self.admin_url = None
-        self.region_id = None
-        self.endpoints = {}
+        self.bootstrapper = bootstrap.Bootstrapper()
 
     @classmethod
     def add_argument_parser(cls, subparsers):
@@ -106,17 +94,17 @@ class BootStrap(BaseApp):
                             metavar='OS_BOOTSTRAP_ADMIN_URL',
                             help=('The initial identity admin url created '
                                   'during the keystone bootstrap process. '
-                                  'e.g. http://127.0.0.1:35357/v2.0'))
+                                  'e.g. http://127.0.0.1:35357/v3'))
         parser.add_argument('--bootstrap-public-url',
                             metavar='OS_BOOTSTRAP_PUBLIC_URL',
                             help=('The initial identity public url created '
                                   'during the keystone bootstrap process. '
-                                  'e.g. http://127.0.0.1:5000/v2.0'))
+                                  'e.g. http://127.0.0.1:5000/v3'))
         parser.add_argument('--bootstrap-internal-url',
                             metavar='OS_BOOTSTRAP_INTERNAL_URL',
                             help=('The initial identity internal url created '
                                   'during the keystone bootstrap process. '
-                                  'e.g. http://127.0.0.1:5000/v2.0'))
+                                  'e.g. http://127.0.0.1:5000/v3'))
         parser.add_argument('--bootstrap-region-id',
                             metavar='OS_BOOTSTRAP_REGION_ID',
                             help=('The initial region_id endpoints will be '
@@ -124,15 +112,14 @@ class BootStrap(BaseApp):
                                   'process.'))
         return parser
 
-    def load_backends(self):
-        drivers = backends.load_backends()
-        self.resource_manager = drivers['resource_api']
-        self.identity_manager = drivers['identity_api']
-        self.assignment_manager = drivers['assignment_api']
-        self.catalog_manager = drivers['catalog_api']
-        self.role_manager = drivers['role_api']
+    def do_bootstrap(self):
+        """Perform the bootstrap actions.
 
-    def _get_config(self):
+        Create bootstrap user, project, and role so that CMS, humans, or
+        scripts can continue to perform initial setup (domains, projects,
+        services, endpoints, etc) of Keystone when standing up a new
+        deployment.
+        """
         self.username = (
             os.environ.get('OS_BOOTSTRAP_USERNAME') or
             CONF.command.bootstrap_username)
@@ -160,202 +147,30 @@ class BootStrap(BaseApp):
         self.region_id = (
             os.environ.get('OS_BOOTSTRAP_REGION_ID') or
             CONF.command.bootstrap_region_id)
-
-    def do_bootstrap(self):
-        """Perform the bootstrap actions.
-
-        Create bootstrap user, project, and role so that CMS, humans, or
-        scripts can continue to perform initial setup (domains, projects,
-        services, endpoints, etc) of Keystone when standing up a new
-        deployment.
-        """
-        self._get_config()
+        self.service_id = None
+        self.endpoints = None
 
         if self.password is None:
             print(_('Either --bootstrap-password argument or '
                     'OS_BOOTSTRAP_PASSWORD must be set.'))
             raise ValueError
 
-        # NOTE(morganfainberg): Ensure the default domain is in-fact created
-        default_domain = {
-            'id': CONF.identity.default_domain_id,
-            'name': 'Default',
-            'enabled': True,
-            'description': 'The default domain'
-        }
-        try:
-            self.resource_manager.create_domain(
-                domain_id=default_domain['id'],
-                domain=default_domain)
-            LOG.info(_LI('Created domain %s'), default_domain['id'])
-        except exception.Conflict:
-            # NOTE(morganfainberg): Domain already exists, continue on.
-            LOG.info(_LI('Domain %s already exists, skipping creation.'),
-                     default_domain['id'])
+        self.bootstrapper.admin_password = self.password
+        self.bootstrapper.admin_username = self.username
+        self.bootstrapper.project_name = self.project_name
+        self.bootstrapper.admin_role_name = self.role_name
+        self.bootstrapper.service_name = self.service_name
+        self.bootstrapper.service_id = self.service_id
+        self.bootstrapper.admin_url = self.admin_url
+        self.bootstrapper.public_url = self.public_url
+        self.bootstrapper.internal_url = self.internal_url
+        self.bootstrapper.region_id = self.region_id
 
-        try:
-            self.resource_manager.create_project(
-                project_id=self.project_id,
-                project={'enabled': True,
-                         'id': self.project_id,
-                         'domain_id': default_domain['id'],
-                         'description': 'Bootstrap project for initializing '
-                                        'the cloud.',
-                         'name': self.project_name}
-            )
-            LOG.info(_LI('Created project %s'), self.project_name)
-        except exception.Conflict:
-            LOG.info(_LI('Project %s already exists, skipping creation.'),
-                     self.project_name)
-            project = self.resource_manager.get_project_by_name(
-                self.project_name, default_domain['id'])
-            self.project_id = project['id']
-
-        # NOTE(morganfainberg): Do not create the user if it already exists.
-        try:
-            user = self.identity_manager.get_user_by_name(self.username,
-                                                          default_domain['id'])
-            LOG.info(_LI('User %s already exists, skipping creation.'),
-                     self.username)
-
-            # Remember whether the user was enabled or not, so that we can
-            # provide useful logging output later.
-            was_enabled = user['enabled']
-
-            # To keep bootstrap idempotent, try to reset the user's password
-            # and ensure that they are enabled. This allows bootstrap to act as
-            # a recovery tool, without having to create a new user.
-            user = self.identity_manager.update_user(
-                user['id'],
-                {'enabled': True,
-                 'password': self.password})
-            LOG.info(_LI('Reset password for user %s.'), self.username)
-            if not was_enabled and user['enabled']:
-                # Although we always try to enable the user, this log message
-                # only makes sense if we know that the user was previously
-                # disabled.
-                LOG.info(_LI('Enabled user %s.'), self.username)
-        except exception.UserNotFound:
-            user = self.identity_manager.create_user(
-                user_ref={'name': self.username,
-                          'enabled': True,
-                          'domain_id': default_domain['id'],
-                          'password': self.password
-                          }
-            )
-            LOG.info(_LI('Created user %s'), self.username)
-
-        # NOTE(morganfainberg): Do not create the role if it already exists.
-        try:
-            self.role_manager.create_role(
-                role_id=self.role_id,
-                role={'name': self.role_name,
-                      'id': self.role_id},
-            )
-            LOG.info(_LI('Created role %s'), self.role_name)
-        except exception.Conflict:
-            LOG.info(_LI('Role %s exists, skipping creation.'), self.role_name)
-            # NOTE(davechen): There is no backend method to get the role
-            # by name, so build the hints to list the roles and filter by
-            # name instead.
-            hints = driver_hints.Hints()
-            hints.add_filter('name', self.role_name)
-            role = self.role_manager.list_roles(hints)
-            self.role_id = role[0]['id']
-
-        # NOTE(morganfainberg): Handle the case that the role assignment has
-        # already occurred.
-        try:
-            self.assignment_manager.add_role_to_user_and_project(
-                user_id=user['id'],
-                tenant_id=self.project_id,
-                role_id=self.role_id
-            )
-            LOG.info(_LI('Granted %(role)s on %(project)s to user'
-                         ' %(username)s.'),
-                     {'role': self.role_name,
-                      'project': self.project_name,
-                      'username': self.username})
-        except exception.Conflict:
-            LOG.info(_LI('User %(username)s already has %(role)s on '
-                         '%(project)s.'),
-                     {'username': self.username,
-                      'role': self.role_name,
-                      'project': self.project_name})
-
-        if self.region_id:
-            try:
-                self.catalog_manager.create_region(
-                    region_ref={'id': self.region_id}
-                )
-                LOG.info(_LI('Created region %s'), self.region_id)
-            except exception.Conflict:
-                LOG.info(_LI('Region %s exists, skipping creation.'),
-                         self.region_id)
-
-        if self.public_url or self.admin_url or self.internal_url:
-            hints = driver_hints.Hints()
-            hints.add_filter('type', 'identity')
-            services = self.catalog_manager.list_services(hints)
-
-            if services:
-                service_ref = services[0]
-
-                hints = driver_hints.Hints()
-                hints.add_filter('service_id', service_ref['id'])
-                if self.region_id:
-                    hints.add_filter('region_id', self.region_id)
-
-                endpoints = self.catalog_manager.list_endpoints(hints)
-            else:
-                service_ref = {'id': uuid.uuid4().hex,
-                               'name': self.service_name,
-                               'type': 'identity',
-                               'enabled': True}
-
-                self.catalog_manager.create_service(
-                    service_id=service_ref['id'],
-                    service_ref=service_ref)
-
-                endpoints = []
-
-            self.service_id = service_ref['id']
-
-            available_interfaces = {e['interface']: e for e in endpoints}
-            expected_endpoints = {'public': self.public_url,
-                                  'internal': self.internal_url,
-                                  'admin': self.admin_url}
-
-            for interface, url in expected_endpoints.items():
-                if not url:
-                    # not specified to bootstrap command
-                    continue
-
-                try:
-                    endpoint_ref = available_interfaces[interface]
-                except KeyError:
-                    endpoint_ref = {'id': uuid.uuid4().hex,
-                                    'interface': interface,
-                                    'url': url,
-                                    'service_id': self.service_id,
-                                    'enabled': True}
-
-                    if self.region_id:
-                        endpoint_ref['region_id'] = self.region_id
-
-                    self.catalog_manager.create_endpoint(
-                        endpoint_id=endpoint_ref['id'],
-                        endpoint_ref=endpoint_ref)
-
-                    LOG.info(_LI('Created %(interface)s endpoint %(url)s'),
-                             {'interface': interface, 'url': url})
-                else:
-                    # NOTE(jamielennox): electing not to update existing
-                    # endpoints here. There may be call to do so in future.
-                    LOG.info(_LI('Skipping %s endpoint as already created'),
-                             interface)
-
-                self.endpoints[interface] = endpoint_ref['id']
+        self.bootstrapper.bootstrap()
+        self.reader_role_id = self.bootstrapper.reader_role_id
+        self.member_role_id = self.bootstrapper.member_role_id
+        self.role_id = self.bootstrapper.admin_role_id
+        self.project_id = self.bootstrapper.project_id
 
     @classmethod
     def main(cls):
@@ -428,20 +243,88 @@ class DbSync(BaseApp):
                                  'that are no longer required. This command '
                                  'should be run after all keystone nodes are '
                                  'running the new release.'))
+
+        group.add_argument('--check', default=False, action='store_true',
+                           help=('Check for outstanding database actions that '
+                                 'still need to be executed. This command can '
+                                 'be used to verify the condition of the '
+                                 'current database state.'))
         return parser
 
+    @classmethod
+    def check_db_sync_status(self):
+        status = 0
+        try:
+            expand_version = upgrades.get_db_version(repo='expand_repo')
+        except migration.exception.DBMigrationError:
+            LOG.info('Your database is not currently under version '
+                     'control or the database is already controlled. Your '
+                     'first step is to run `keystone-manage db_sync '
+                     '--expand`.')
+            return 2
+        try:
+            migrate_version = upgrades.get_db_version(
+                repo='data_migration_repo')
+        except migration.exception.DBMigrationError:
+            migrate_version = 0
+        try:
+            contract_version = upgrades.get_db_version(repo='contract_repo')
+        except migration.exception.DBMigrationError:
+            contract_version = 0
+
+        repo = migrate.versioning.repository.Repository(
+            upgrades.find_repo('expand_repo'))
+        migration_script_version = int(max(repo.versions.versions))
+
+        if (contract_version > migrate_version or migrate_version >
+                expand_version):
+            LOG.info('Your database is out of sync. For more information '
+                     'refer to https://docs.openstack.org/keystone/'
+                     'latest/admin/identity-upgrading.html')
+            status = 1
+        elif migration_script_version > expand_version:
+            LOG.info('Your database is not up to date. Your first step is '
+                     'to run `keystone-manage db_sync --expand`.')
+            status = 2
+        elif expand_version > migrate_version:
+            LOG.info('Expand version is ahead of migrate. Your next step '
+                     'is to run `keystone-manage db_sync --migrate`.')
+            status = 3
+        elif migrate_version > contract_version:
+            LOG.info('Migrate version is ahead of contract. Your next '
+                     'step is to run `keystone-manage db_sync --contract`.')
+            status = 4
+        elif (migration_script_version == expand_version == migrate_version ==
+                contract_version):
+            LOG.info('All db_sync commands are upgraded to the same '
+                     'version and up-to-date.')
+        LOG.info('The latest installed migration script version is: '
+                 '%(script)d.\nCurrent repository versions:\nExpand: '
+                 '%(expand)d \nMigrate: %(migrate)d\nContract: '
+                 '%(contract)d', {'script': migration_script_version,
+                                  'expand': expand_version,
+                                  'migrate': migrate_version,
+                                  'contract': contract_version})
+        return status
+
     @staticmethod
     def main():
         assert_not_extension(CONF.command.extension)
-
-        if CONF.command.expand:
-            migration_helpers.expand_schema()
+        # It is possible to run expand and migrate at the same time,
+        # expand needs to run first however.
+        if CONF.command.check:
+            sys.exit(DbSync.check_db_sync_status())
+        elif CONF.command.expand and CONF.command.migrate:
+            upgrades.expand_schema()
+            upgrades.migrate_data()
+        elif CONF.command.expand:
+            upgrades.expand_schema()
         elif CONF.command.migrate:
-            migration_helpers.migrate_data()
+            upgrades.migrate_data()
         elif CONF.command.contract:
-            migration_helpers.contract_schema()
+            upgrades.contract_schema()
         else:
-            migration_helpers.offline_sync_database_to_version(
+            upgrades.offline_sync_database_to_version(
                 CONF.command.version)
 
 
@@ -464,7 +347,7 @@ class DbVersion(BaseApp):
     @staticmethod
     def main():
         assert_not_extension(CONF.command.extension)
-        migration_helpers.print_db_version()
+        print(upgrades.get_db_version())
 
 
 class BasePermissionsSetup(BaseApp):
@@ -501,44 +384,6 @@ class BasePermissionsSetup(BaseApp):
         return keystone_user_id, keystone_group_id
 
 
-class BaseCertificateSetup(BasePermissionsSetup):
-    """Provides common options for certificate setup."""
-
-    @classmethod
-    def add_argument_parser(cls, subparsers):
-        parser = super(BaseCertificateSetup,
-                       cls).add_argument_parser(subparsers)
-        parser.add_argument('--rebuild', default=False, action='store_true',
-                            help=('Rebuild certificate files: erase previous '
-                                  'files and regenerate them.'))
-        return parser
-
-
-class PKISetup(BaseCertificateSetup):
-    """Set up Key pairs and certificates for token signing and verification.
-
-    This is NOT intended for production use, see Keystone Configuration
-    documentation for details. As of the Mitaka release, this command has
-    been DEPRECATED and may be removed in the 'O' release.
-    """
-
-    name = 'pki_setup'
-
-    @classmethod
-    def main(cls):
-        versionutils.report_deprecated_feature(
-            LOG,
-            _LW("keystone-manage pki_setup is deprecated as of Mitaka in "
-                "favor of not using PKI tokens and may be removed in 'O' "
-                "release."))
-        LOG.warning(_LW('keystone-manage pki_setup is not recommended for '
-                        'production use.'))
-        keystone_user_id, keystone_group_id = cls.get_user_group()
-        conf_pki = openssl.ConfigurePKI(keystone_user_id, keystone_group_id,
-                                        rebuild=CONF.command.rebuild)
-        conf_pki.run()
-
-
 class FernetSetup(BasePermissionsSetup):
     """Setup a key repository for Fernet tokens.
 
@@ -552,16 +397,16 @@ class FernetSetup(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.fernet_tokens.key_repository,
-            CONF.fernet_tokens.max_active_keys
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
         )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        fernet_utils.create_key_directory(keystone_user_id, keystone_group_id)
-        if fernet_utils.validate_key_repository(requires_write=True):
-            fernet_utils.initialize_key_repository(
+        futils.create_key_directory(keystone_user_id, keystone_group_id)
+        if futils.validate_key_repository(requires_write=True):
+            futils.initialize_key_repository(
                 keystone_user_id, keystone_group_id)
 
 
@@ -587,15 +432,76 @@ class FernetRotate(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.fernet_tokens.key_repository,
-            CONF.fernet_tokens.max_active_keys
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
         )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        if fernet_utils.validate_key_repository(requires_write=True):
-            fernet_utils.rotate_keys(keystone_user_id, keystone_group_id)
+        if futils.validate_key_repository(requires_write=True):
+            futils.rotate_keys(keystone_user_id, keystone_group_id)
+
+
+class TokenSetup(BasePermissionsSetup):
+    """Setup a key repository for tokens.
+
+    This also creates a primary key used for both creating and validating
+    tokens. To improve security, you should rotate your keys (using
+    keystone-manage token_rotate, for example).
+
+    """
+
+    name = 'token_setup'
+
+    @classmethod
+    def main(cls):
+        futils = fernet_utils.FernetUtils(
+            # TODO(gagehugo) Change this to CONF.token
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
+        )
+
+        keystone_user_id, keystone_group_id = cls.get_user_group()
+        futils.create_key_directory(keystone_user_id, keystone_group_id)
+        if futils.validate_key_repository(requires_write=True):
+            futils.initialize_key_repository(
+                keystone_user_id, keystone_group_id)
+
+
+class TokenRotate(BasePermissionsSetup):
+    """Rotate token encryption keys.
+
+    This assumes you have already run keystone-manage token_setup.
+
+    A new primary key is placed into rotation, which is used for new tokens.
+    The old primary key is demoted to secondary, which can then still be used
+    for validating tokens. Excess secondary keys (beyond [token]
+    max_active_keys) are revoked. Revoked keys are permanently deleted. A new
+    staged key will be created and used to validate tokens. The next time key
+    rotation takes place, the staged key will be put into rotation as the
+    primary key.
+
+    Rotating keys too frequently, or with [token] max_active_keys set
+    too low, will cause tokens to become invalid prior to their expiration.
+
+    """
+
+    name = 'token_rotate'
+
+    @classmethod
+    def main(cls):
+        futils = fernet_utils.FernetUtils(
+            # TODO(gagehugo) Change this to CONF.token
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
+        )
+
+        keystone_user_id, keystone_group_id = cls.get_user_group()
+        if futils.validate_key_repository(requires_write=True):
+            futils.rotate_keys(keystone_user_id, keystone_group_id)
 
 
 class CredentialSetup(BasePermissionsSetup):
@@ -611,16 +517,16 @@ class CredentialSetup(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.credential.key_repository,
-            credential_fernet.MAX_ACTIVE_KEYS
+            credential_fernet.MAX_ACTIVE_KEYS,
+            'credential'
         )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        fernet_utils.create_key_directory(keystone_user_id, keystone_group_id)
-        if fernet_utils.validate_key_repository(requires_write=True):
-            fernet_utils.initialize_key_repository(
+        futils.create_key_directory(keystone_user_id, keystone_group_id)
+        if futils.validate_key_repository(requires_write=True):
+            futils.initialize_key_repository(
                 keystone_user_id,
                 keystone_group_id
             )
@@ -683,17 +589,17 @@ class CredentialRotate(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.credential.key_repository,
-            credential_fernet.MAX_ACTIVE_KEYS
+            credential_fernet.MAX_ACTIVE_KEYS,
+            'credential'
         )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        if fernet_utils.validate_key_repository(requires_write=True):
+        if futils.validate_key_repository(requires_write=True):
             klass = cls()
             klass.validate_primary_key()
-            fernet_utils.rotate_keys(keystone_user_id, keystone_group_id)
+            futils.rotate_keys(keystone_user_id, keystone_group_id)
 
 
 class CredentialMigrate(BasePermissionsSetup):
@@ -743,12 +649,12 @@ class CredentialMigrate(BasePermissionsSetup):
     @classmethod
     def main(cls):
         # Check to make sure we have a repository that works...
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.credential.key_repository,
-            credential_fernet.MAX_ACTIVE_KEYS
+            credential_fernet.MAX_ACTIVE_KEYS,
+            'credential'
         )
-        fernet_utils.validate_key_repository(requires_write=True)
+        futils.validate_key_repository(requires_write=True)
         klass = cls()
         klass.migrate_credentials()
 
@@ -760,15 +666,14 @@ class TokenFlush(BaseApp):
 
     @classmethod
     def main(cls):
-        token_manager = token.persistence.PersistenceManager()
-        try:
-            token_manager.flush_expired_tokens()
-        except exception.NotImplemented:
-            # NOTE(ravelar159): Stop NotImplemented from unsupported token
-            # driver when using token_flush and print out warning instead
-            LOG.warning(_LW('Token driver %s does not support token_flush. '
-                            'The token_flush command had no effect.'),
-                        CONF.token.driver)
+        LOG.warning(
+            'The token_flush command is deprecated and no longer needed with '
+            'the development of non-persistent token formats. It will be '
+            'removed in Stein. It is recommended that you remove usage of '
+            'this command or integrate it\'s functionality into a separate '
+            'tool if you are using an out-of-tree provider that relies on '
+            'persistent token storage.'
+        )
 
 
 class MappingPurge(BaseApp):
@@ -846,7 +751,7 @@ class MappingPurge(BaseApp):
         if CONF.command.local_id is not None:
             mapping['local_id'] = CONF.command.local_id
         if CONF.command.type is not None:
-            mapping['type'] = CONF.command.type
+            mapping['entity_type'] = CONF.command.type
 
         mapping_manager.purge_mappings(mapping)
 
@@ -863,7 +768,7 @@ def _domain_config_finder(conf_dir):
 
     :returns: generator yielding (filename, domain_name) tuples
     """
-    LOG.info(_LI('Scanning %r for domain config files'), conf_dir)
+    LOG.info('Scanning %r for domain config files', conf_dir)
     for r, d, f in os.walk(conf_dir):
         for fname in f:
             if (fname.startswith(DOMAIN_CONF_FHEAD) and
@@ -874,8 +779,8 @@ def _domain_config_finder(conf_dir):
                     yield (os.path.join(r, fname), domain_name)
                     continue
 
-            LOG.warning(_LW('Ignoring file (%s) while scanning '
-                            'domain config directory'), fname)
+            LOG.warning('Ignoring file (%s) while scanning '
+                        'domain config directory', fname)
 
 
 class DomainConfigUploadFiles(object):
@@ -963,8 +868,8 @@ class DomainConfigUploadFiles(object):
                                                      sections)
             return True
         except Exception as e:
-            msg = _LE('Error processing config file for domain: '
-                      '%(domain_name)s, file: %(filename)s, error: %(error)s')
+            msg = ('Error processing config file for domain: '
+                   '%(domain_name)s, file: %(filename)s, error: %(error)s')
             LOG.error(msg,
                       {'domain_name': domain_name,
                        'filename': file_name,
@@ -995,9 +900,21 @@ class DomainConfigUploadFiles(object):
                 return False
             return True
 
+        success_cnt = 0
+        failure_cnt = 0
         for filename, domain_name in self._domain_config_finder(conf_dir):
-            self._upload_config_to_database(filename, domain_name)
+            if self._upload_config_to_database(filename, domain_name):
+                success_cnt += 1
+                LOG.info('Successfully uploaded domain config %r',
+                         filename)
+            else:
+                failure_cnt += 1
 
+        if success_cnt == 0:
+            LOG.warning('No domain configs uploaded from %r', conf_dir)
+
+        if failure_cnt:
+            return False
         return True
 
     def run(self):
@@ -1040,11 +957,6 @@ class DomainConfigUpload(BaseApp):
 
     @staticmethod
     def main():
-        versionutils.report_deprecated_feature(
-            LOG,
-            _LW("keystone-manage domain_config_upload is deprecated as of "
-                "Newton in favor of setting domain config options via the API "
-                "and may be removed in 'P' release."))
         dcu = DomainConfigUploadFiles()
         status = dcu.run()
         if status is not None:
@@ -1059,7 +971,7 @@ class SamlIdentityProviderMetadata(BaseApp):
     @staticmethod
     def main():
         metadata = idp.MetadataGenerator().generate_metadata()
-        print(metadata.to_string())
+        print(metadata)
 
 
 class MappingEngineTester(BaseApp):
@@ -1067,65 +979,89 @@ class MappingEngineTester(BaseApp):
 
     name = 'mapping_engine'
 
-    @staticmethod
-    def read_rules(path):
+    def __init__(self):
+        super(MappingEngineTester, self).__init__()
+        self.mapping_id = uuid.uuid4().hex
+        self.rules_pathname = None
+        self.rules = None
+        self.assertion_pathname = None
+        self.assertion = None
+
+    def read_rules(self, path):
+        self.rules_pathname = path
         try:
             with open(path) as file:
-                return jsonutils.load(file)
+                self.rules = jsonutils.load(file)
         except ValueError as e:
             raise SystemExit(_('Error while parsing rules '
                                '%(path)s: %(err)s') % {'path': path, 'err': e})
 
-    @staticmethod
-    def read_file(path):
+    def read_assertion(self, path):
+        self.assertion_pathname = path
         try:
             with open(path) as file:
-                return file.read().strip()
+                self.assertion = file.read().strip()
         except IOError as e:
             raise SystemExit(_("Error while opening file "
                                "%(path)s: %(err)s") % {'path': path, 'err': e})
 
-    @staticmethod
-    def normalize_assertion(assertion):
-        def split(line):
+    def normalize_assertion(self):
+        def split(line, line_num):
             try:
                 k, v = line.split(':', 1)
                 return k.strip(), v.strip()
-            except ValueError as e:
-                msg = _("Error while parsing line: '%(line)s': %(err)s")
-                raise SystemExit(msg % {'line': line, 'err': e})
-        assertion = assertion.split('\n')
+            except ValueError:
+                msg = _("assertion file %(pathname)s at line %(line_num)d "
+                        "expected 'key: value' but found '%(line)s' "
+                        "see help for file format")
+                raise SystemExit(msg % {'pathname': self.assertion_pathname,
+                                        'line_num': line_num,
+                                        'line': line})
+        assertion = self.assertion.split('\n')
         assertion_dict = {}
         prefix = CONF.command.prefix
-        for line in assertion:
-            k, v = split(line)
+        for line_num, line in enumerate(assertion, 1):
+            line = line.strip()
+            if line == '':
+                continue
+            k, v = split(line, line_num)
             if prefix:
                 if k.startswith(prefix):
                     assertion_dict[k] = v
             else:
                 assertion_dict[k] = v
-        return assertion_dict
+        self.assertion = assertion_dict
 
-    @staticmethod
-    def normalize_rules(rules):
-        if isinstance(rules, list):
-            return {'rules': rules}
-        else:
-            return rules
+    def normalize_rules(self):
+        if isinstance(self.rules, list):
+            self.rules = {'rules': self.rules}
 
     @classmethod
     def main(cls):
-        if not CONF.command.engine_debug:
+        if CONF.command.engine_debug:
+            mapping_engine.LOG.logger.setLevel('DEBUG')
+        else:
             mapping_engine.LOG.logger.setLevel('WARN')
 
-        rules = MappingEngineTester.read_rules(CONF.command.rules)
-        rules = MappingEngineTester.normalize_rules(rules)
-        mapping_engine.validate_mapping_structure(rules)
+        tester = cls()
 
-        assertion = MappingEngineTester.read_file(CONF.command.input)
-        assertion = MappingEngineTester.normalize_assertion(assertion)
-        rp = mapping_engine.RuleProcessor(rules['rules'])
-        print(jsonutils.dumps(rp.process(assertion), indent=2))
+        tester.read_rules(CONF.command.rules)
+        tester.normalize_rules()
+        mapping_engine.validate_mapping_structure(tester.rules)
+
+        tester.read_assertion(CONF.command.input)
+        tester.normalize_assertion()
+
+        if CONF.command.engine_debug:
+            print("Using Rules:\n%s" % (
+                jsonutils.dumps(tester.rules, indent=2)))
+            print("Using Assertion:\n%s" % (
+                jsonutils.dumps(tester.assertion, indent=2)))
+
+        rp = mapping_engine.RuleProcessor(tester.mapping_id,
+                                          tester.rules['rules'])
+        mapped = rp.process(tester.assertion)
+        print(jsonutils.dumps(mapped, indent=2))
 
     @classmethod
     def add_argument_parser(cls, subparsers):
@@ -1196,8 +1132,8 @@ class MappingPopulate(BaseApp):
         try:
             domain_id = cls.resource_api.get_domain_by_name(domain_name)['id']
         except exception.DomainNotFound:
-            print(_('Invalid domain name or ID: %(domain)s') % {
-                'domain': domain_id})
+            print(_('Invalid domain name: %(domain)s') % {
+                'domain': domain_name})
             return False
         # We don't actually need to tackle id_mapping_api in order to get
         # entries there, because list_users does this anyway. That's why it
@@ -1219,9 +1155,10 @@ CMDS = [
     MappingPopulate,
     MappingPurge,
     MappingEngineTester,
-    PKISetup,
     SamlIdentityProviderMetadata,
     TokenFlush,
+    TokenRotate,
+    TokenSetup
 ]
 
 
@@ -1236,19 +1173,44 @@ command_opt = cfg.SubCommandOpt('command',
                                 handler=add_command_parsers)
 
 
-def main(argv=None, config_files=None):
+def main(argv=None, developer_config_file=None):
+    """Main entry point into the keystone-manage CLI utility.
+
+    :param argv: Arguments supplied via the command line using the ``sys``
+                 standard library.
+    :type argv: list
+    :param developer_config_file: The location of a configuration file normally
+                                  found in development environments.
+    :type developer_config_file: string
+
+    """
     CONF.register_cli_opt(command_opt)
 
     keystone.conf.configure()
     sql.initialize()
     keystone.conf.set_default_for_default_log_levels()
 
+    user_supplied_config_file = False
+    if argv:
+        for argument in argv:
+            if argument == '--config-file':
+                user_supplied_config_file = True
+
+    if developer_config_file:
+        developer_config_file = [developer_config_file]
+
+    # NOTE(lbragstad): At this point in processing, the first element of argv
+    # is the binary location of keystone-manage, which oslo.config doesn't need
+    # and is keystone specific. Only pass a list of arguments so that
+    # oslo.config can determine configuration file locations based on user
+    # provided arguments, if present.
     CONF(args=argv[1:],
          project='keystone',
          version=pbr.version.VersionInfo('keystone').version_string(),
          usage='%(prog)s [' + '|'.join([cmd.name for cmd in CMDS]) + ']',
-         default_config_files=config_files)
-    if not CONF.default_config_files:
-        LOG.warning(_LW('Config file not found, using default configs.'))
+         default_config_files=developer_config_file)
+
+    if not CONF.default_config_files and not user_supplied_config_file:
+        LOG.warning('Config file not found, using default configs.')
     keystone.conf.setup_logging()
     CONF.command.cmd_class.main()
