@@ -18,7 +18,6 @@ from cryptography import fernet
 from oslo_log import log
 
 import keystone.conf
-from keystone.i18n import _LE, _LW, _LI
 
 
 LOG = log.getLogger(__name__)
@@ -36,9 +35,11 @@ NULL_KEY = base64.urlsafe_b64encode(b'\x00' * 32)
 
 class FernetUtils(object):
 
-    def __init__(self, key_repository=None, max_active_keys=None):
+    def __init__(self, key_repository=None, max_active_keys=None,
+                 config_group=None):
         self.key_repository = key_repository
         self.max_active_keys = max_active_keys
+        self.config_group = config_group
 
     def validate_key_repository(self, requires_write=False):
         """Validate permissions on the key repository directory."""
@@ -54,16 +55,18 @@ class FernetUtils(object):
 
         if not is_valid:
             LOG.error(
-                _LE('Either [fernet_tokens] key_repository does not exist or '
-                    'Keystone does not have sufficient permission to access '
-                    'it: %s'), self.key_repository)
+                'Either [%(config_group)s] key_repository does not exist '
+                'or Keystone does not have sufficient permission to '
+                'access it: %(key_repo)s',
+                {'key_repo': self.key_repository,
+                 'config_group': self.config_group})
         else:
             # ensure the key repository isn't world-readable
             stat_info = os.stat(self.key_repository)
             if(stat_info.st_mode & stat.S_IROTH or
                stat_info.st_mode & stat.S_IXOTH):
-                LOG.warning(_LW(
-                    'key_repository is world readable: %s'),
+                LOG.warning(
+                    'key_repository is world readable: %s',
                     self.key_repository)
 
         return is_valid
@@ -72,17 +75,17 @@ class FernetUtils(object):
                              keystone_group_id=None):
         """Attempt to create the key directory if it doesn't exist."""
         if not os.access(self.key_repository, os.F_OK):
-            LOG.info(_LI(
+            LOG.info(
                 'key_repository does not appear to exist; attempting to '
-                'create it'))
+                'create it')
 
             try:
                 os.makedirs(self.key_repository, 0o700)
             except OSError:
-                LOG.error(_LE(
+                LOG.error(
                     'Failed to create key_repository: either it already '
                     'exists or you don\'t have sufficient permissions to '
-                    'create it'))
+                    'create it')
 
             if keystone_user_id and keystone_group_id:
                 os.chown(
@@ -90,16 +93,27 @@ class FernetUtils(object):
                     keystone_user_id,
                     keystone_group_id)
             elif keystone_user_id or keystone_group_id:
-                LOG.warning(_LW(
+                LOG.warning(
                     'Unable to change the ownership of key_repository without '
                     'a keystone user ID and keystone group ID both being '
-                    'provided: %s') % self.key_repository)
+                    'provided: %s', self.key_repository)
 
     def _create_new_key(self, keystone_user_id, keystone_group_id):
         """Securely create a new encryption key.
 
         Create a new key that is readable by the Keystone group and Keystone
         user.
+
+        To avoid disk write failure, this function will create a tmp key file
+        first, and then rename it as the valid new key.
+        """
+        self._create_tmp_new_key(keystone_user_id, keystone_group_id)
+        self._become_valid_new_key()
+
+    def _create_tmp_new_key(self, keystone_user_id, keystone_group_id):
+        """Securely create a new tmp encryption key.
+
+        This created key is not effective until _become_valid_new_key().
         """
         key = fernet.Fernet.generate_key()  # key is bytes
 
@@ -111,17 +125,22 @@ class FernetUtils(object):
             os.setegid(keystone_group_id)
             os.seteuid(keystone_user_id)
         elif keystone_user_id or keystone_group_id:
-            LOG.warning(_LW(
+            LOG.warning(
                 'Unable to change the ownership of the new key without a '
                 'keystone user ID and keystone group ID both being provided: '
-                '%s') %
-                self.key_repository)
+                '%s', self.key_repository)
         # Determine the file name of the new key
-        key_file = os.path.join(self.key_repository, '0')
+        key_file = os.path.join(self.key_repository, '0.tmp')
+        create_success = False
         try:
             with open(key_file, 'w') as f:
                 # convert key to str for the file.
                 f.write(key.decode('utf-8'))
+                f.flush()
+                create_success = True
+        except IOError:
+            LOG.error('Failed to create new temporary key: %s', key_file)
+            raise
         finally:
             # After writing the key, set the umask back to it's original value.
             # Do the same with group and user identifiers if a Keystone group
@@ -130,8 +149,44 @@ class FernetUtils(object):
             if keystone_user_id and keystone_group_id:
                 os.seteuid(old_euid)
                 os.setegid(old_egid)
+            # Deal with the tmp key file
+            if not create_success and os.access(key_file, os.F_OK):
+                os.remove(key_file)
 
-        LOG.info(_LI('Created a new key: %s'), key_file)
+        LOG.info('Created a new temporary key: %s', key_file)
+
+    def _become_valid_new_key(self):
+        """Make the tmp new key a valid new key.
+
+        The tmp new key must be created by _create_tmp_new_key().
+        """
+        tmp_key_file = os.path.join(self.key_repository, '0.tmp')
+        valid_key_file = os.path.join(self.key_repository, '0')
+
+        os.rename(tmp_key_file, valid_key_file)
+
+        LOG.info('Become a valid new key: %s', valid_key_file)
+
+    def _get_key_files(self, key_repo):
+        key_files = dict()
+        keys = dict()
+        for filename in os.listdir(key_repo):
+            path = os.path.join(key_repo, str(filename))
+            if os.path.isfile(path):
+                with open(path, 'r') as key_file:
+                    try:
+                        key_id = int(filename)
+                    except ValueError:  # nosec : name is not a number
+                        pass
+                    else:
+                        key = key_file.read()
+                        if len(key) == 0:
+                            LOG.warning('Ignoring empty key found in key '
+                                        'repository: %s', path)
+                            continue
+                        key_files[key_id] = path
+                        keys[key_id] = key
+        return key_files, keys
 
     def initialize_key_repository(self, keystone_user_id=None,
                                   keystone_group_id=None):
@@ -144,7 +199,7 @@ class FernetUtils(object):
         # make sure we have work to do before proceeding
         if os.access(os.path.join(self.key_repository, '0'),
                      os.F_OK):
-            LOG.info(_LI('Key repository is already initialized; aborting.'))
+            LOG.info('Key repository is already initialized; aborting.')
             return
 
         # bootstrap an existing key
@@ -174,27 +229,21 @@ class FernetUtils(object):
 
         """
         # read the list of key files
-        key_files = dict()
-        for filename in os.listdir(self.key_repository):
-            path = os.path.join(self.key_repository, str(filename))
-            if os.path.isfile(path):
-                try:
-                    key_id = int(filename)
-                except ValueError:  # nosec : name isn't a number
-                    pass
-                else:
-                    key_files[key_id] = path
+        key_files, _ = self._get_key_files(self.key_repository)
 
-        LOG.info(_LI('Starting key rotation with %(count)s key files: '
-                     '%(list)s'), {
-            'count': len(key_files),
-            'list': list(key_files.values())})
+        LOG.info('Starting key rotation with %(count)s key files: '
+                 '%(list)s', {
+                     'count': len(key_files),
+                     'list': list(key_files.values())})
+
+        # add a tmp new key to the rotation, which will be the *next* primary
+        self._create_tmp_new_key(keystone_user_id, keystone_group_id)
 
         # determine the number of the new primary key
         current_primary_key = max(key_files.keys())
-        LOG.info(_LI('Current primary key is: %s'), current_primary_key)
+        LOG.info('Current primary key is: %s', current_primary_key)
         new_primary_key = current_primary_key + 1
-        LOG.info(_LI('Next primary key will be: %s'), new_primary_key)
+        LOG.info('Next primary key will be: %s', new_primary_key)
 
         # promote the next primary key to be the primary
         os.rename(
@@ -205,10 +254,10 @@ class FernetUtils(object):
         key_files[new_primary_key] = os.path.join(
             self.key_repository,
             str(new_primary_key))
-        LOG.info(_LI('Promoted key 0 to be the primary: %s'), new_primary_key)
+        LOG.info('Promoted key 0 to be the primary: %s', new_primary_key)
 
-        # add a new key to the rotation, which will be the *next* primary
-        self._create_new_key(keystone_user_id, keystone_group_id)
+        # rename the tmp key to the real staged key
+        self._become_valid_new_key()
 
         max_active_keys = self.max_active_keys
 
@@ -220,7 +269,7 @@ class FernetUtils(object):
         while len(keys) > (max_active_keys - 1):
             index_to_purge = keys.pop()
             key_to_purge = key_files[index_to_purge]
-            LOG.info(_LI('Excess key to purge: %s'), key_to_purge)
+            LOG.info('Excess key to purge: %s', key_to_purge)
             os.remove(key_to_purge)
 
     def load_keys(self, use_null_key=False):
@@ -240,18 +289,7 @@ class FernetUtils(object):
             return []
 
         # build a dictionary of key_number:encryption_key pairs
-        keys = dict()
-        for filename in os.listdir(self.key_repository):
-            path = os.path.join(self.key_repository, str(filename))
-            if os.path.isfile(path):
-                with open(path, 'r') as key_file:
-                    try:
-                        key_id = int(filename)
-                    except ValueError:  # nosec : filename isn't a number,
-                        # ignore this file since it's not a key.
-                        pass
-                    else:
-                        keys[key_id] = key_file.read()
+        _, keys = self._get_key_files(self.key_repository)
 
         if len(keys) != self.max_active_keys:
             # Once the number of keys matches max_active_keys, this log entry
@@ -259,14 +297,14 @@ class FernetUtils(object):
             # sense to log this message for tokens since credentials doesn't
             # have a `max_active_key` configuration option.
             if self.key_repository == CONF.fernet_tokens.key_repository:
-                LOG.debug(
-                    'Loaded %(count)d Fernet keys from %(dir)s, but '
-                    '`[fernet_tokens] max_active_keys = %(max)d`; perhaps '
-                    'there have not been enough key rotations to reach '
-                    '`max_active_keys` yet?', {
-                        'count': len(keys),
-                        'max': self.max_active_keys,
-                        'dir': self.key_repository})
+                msg = ('Loaded %(count)d Fernet keys from %(dir)s, but '
+                       '`[fernet_tokens] max_active_keys = %(max)d`; perhaps '
+                       'there have not been enough key rotations to reach '
+                       '`max_active_keys` yet?')
+                LOG.debug(msg, {
+                    'count': len(keys),
+                    'max': self.max_active_keys,
+                    'dir': self.key_repository})
 
         # return the encryption_keys, sorted by key number, descending
         key_list = [keys[x] for x in sorted(keys.keys(), reverse=True)]
