@@ -16,7 +16,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import calendar
 import collections
 import grp
 import hashlib
@@ -30,14 +29,13 @@ from oslo_serialization import jsonutils
 from oslo_utils import reflection
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import passlib.hash
 import six
 from six import moves
 
-from keystone.common import authorization
+from keystone.common import password_hashing
 import keystone.conf
 from keystone import exception
-from keystone.i18n import _, _LE, _LW
+from keystone.i18n import _
 
 
 CONF = keystone.conf.CONF
@@ -46,7 +44,7 @@ WHITELISTED_PROPERTIES = [
     'tenant_id', 'project_id', 'user_id',
     'public_bind_host', 'admin_bind_host',
     'compute_host', 'admin_port', 'public_port',
-    'public_endpoint', 'admin_endpoint', ]
+    'public_endpoint', ]
 
 
 # NOTE(stevermar): This UUID must stay the same, forever, across
@@ -54,6 +52,12 @@ WHITELISTED_PROPERTIES = [
 # used for ID transformation.
 RESOURCE_ID_NAMESPACE = uuid.UUID('4332ecab-770b-4288-a680-b9aca3b1b153')
 
+# Compatibilty for password hashing functions.
+verify_length_and_trunc_password = password_hashing.verify_length_and_trunc_password  # noqa
+hash_password = password_hashing.hash_password
+hash_user_password = password_hashing.hash_user_password
+check_password = password_hashing.check_password
+
 
 def resource_uuid(value):
     """Convert input to valid UUID hex digits."""
@@ -95,30 +99,6 @@ class SmarterEncoder(jsonutils.json.JSONEncoder):
         return super(SmarterEncoder, self).default(obj)
 
 
-class PKIEncoder(SmarterEncoder):
-    """Special encoder to make token JSON a bit shorter."""
-
-    item_separator = ','
-    key_separator = ':'
-
-
-def verify_length_and_trunc_password(password):
-    """Verify and truncate the provided password to the max_password_length."""
-    max_length = CONF.identity.max_password_length
-    try:
-        if len(password) > max_length:
-            if CONF.strict_password_check:
-                raise exception.PasswordVerificationError(size=max_length)
-            else:
-                msg = _LW("Truncating user password to %d characters.")
-                LOG.warning(msg, max_length)
-                return password[:max_length]
-        else:
-            return password
-    except TypeError:
-        raise exception.ValidationError(attribute='string', target='password')
-
-
 def hash_access_key(access):
     hash_ = hashlib.sha256()
     if not isinstance(access, six.binary_type):
@@ -127,35 +107,6 @@ def hash_access_key(access):
     return hash_.hexdigest()
 
 
-def hash_user_password(user):
-    """Hash a user dict's password without modifying the passed-in dict."""
-    password = user.get('password')
-    if password is None:
-        return user
-
-    return dict(user, password=hash_password(password))
-
-
-def hash_password(password):
-    """Hash a password. Hard."""
-    password_utf8 = verify_length_and_trunc_password(password).encode('utf-8')
-    return passlib.hash.sha512_crypt.encrypt(
-        password_utf8, rounds=CONF.crypt_strength)
-
-
-def check_password(password, hashed):
-    """Check that a plaintext password matches hashed.
-
-    hashpw returns the salt value concatenated with the actual hash value.
-    It extracts the actual salt if this value is then passed as the salt.
-
-    """
-    if password is None or hashed is None:
-        return False
-    password_utf8 = verify_length_and_trunc_password(password).encode('utf-8')
-    return passlib.hash.sha512_crypt.verify(password_utf8, hashed)
-
-
 def attr_as_boolean(val_attr):
     """Return the boolean value, decoded from a string.
 
@@ -168,46 +119,21 @@ def attr_as_boolean(val_attr):
     return strutils.bool_from_string(val_attr, default=True)
 
 
-def get_blob_from_credential(credential):
-    try:
-        blob = jsonutils.loads(credential.blob)
-    except (ValueError, TypeError):
-        raise exception.ValidationError(
-            message=_('Invalid blob in credential'))
-    if not blob or not isinstance(blob, dict):
-        raise exception.ValidationError(attribute='blob',
-                                        target='credential')
-    return blob
+def remove_duplicate_dicts_by_id(item_list):
+    """Return a list with duplicate items removed.
 
+    This compares duplicates based on item id. This is to account
+    for dictionaries of items that can contain values, such as a
+    list of tags, that will not be guaranteed to be in order.
 
-def convert_ec2_to_v3_credential(ec2credential):
-    blob = {'access': ec2credential.access,
-            'secret': ec2credential.secret}
-    return {'id': hash_access_key(ec2credential.access),
-            'user_id': ec2credential.user_id,
-            'project_id': ec2credential.tenant_id,
-            'blob': jsonutils.dumps(blob),
-            'type': 'ec2',
-            'extra': jsonutils.dumps({})}
-
-
-def convert_v3_to_ec2_credential(credential):
-    blob = get_blob_from_credential(credential)
-    return {'access': blob.get('access'),
-            'secret': blob.get('secret'),
-            'user_id': credential.user_id,
-            'tenant_id': credential.project_id,
-            }
-
-
-def unixtime(dt_obj):
-    """Format datetime object as unix timestamp.
-
-    :param dt_obj: datetime.datetime object
-    :returns: float
+    :param item_list: a list of dictionaries
+    :returns: a list of unique dictionaries
 
     """
-    return calendar.timegm(dt_obj.utctimetuple())
+    unique = {}
+    for i in item_list:
+        unique[i['id']] = i
+    return list(unique.values())
 
 
 def auth_str_equal(provided, known):
@@ -248,10 +174,10 @@ def setup_remote_pydev_debug():
                             stderrToServer=True)
             return True
         except Exception:
-            LOG.exception(_LE(
+            LOG.exception(
                 'Error setting up the debug environment. Verify that the '
                 'option --debug-url has the format <host>:<port> and that a '
-                'debugger processes is listening on that port.'))
+                'debugger processes is listening on that port.')
             raise
 
 
@@ -372,109 +298,6 @@ def get_unix_group(group=None):
     return group_info.gr_gid, group_info.gr_name
 
 
-def set_permissions(path, mode=None, user=None, group=None, log=None):
-    """Set the ownership and permissions on the pathname.
-
-    Each of the mode, user and group are optional, if None then
-    that aspect is not modified.
-
-    Owner and group may be specified either with a symbolic name
-    or numeric id.
-
-    :param string path: Pathname of directory whose existence is assured.
-    :param object mode: ownership permissions flags (int) i.e. chmod,
-                        if None do not set.
-    :param object user: set user, name (string) or uid (integer),
-                         if None do not set.
-    :param object group: set group, name (string) or gid (integer)
-                         if None do not set.
-    :param logger log: logging.logger object, used to emit log messages,
-                       if None no logging is performed.
-
-    """
-    if user is None:
-        user_uid, user_name = None, None
-    else:
-        user_uid, user_name = get_unix_user(user)
-
-    if group is None:
-        group_gid, group_name = None, None
-    else:
-        group_gid, group_name = get_unix_group(group)
-
-    if log:
-        if mode is None:
-            mode_string = str(mode)
-        else:
-            mode_string = oct(mode)
-        log.debug("set_permissions: "
-                  "path='%s' mode=%s user=%s(%s) group=%s(%s)",
-                  path, mode_string,
-                  user_name, user_uid, group_name, group_gid)
-
-    # Change user and group if specified
-    if user_uid is not None or group_gid is not None:
-        if user_uid is None:
-            user_uid = -1
-        if group_gid is None:
-            group_gid = -1
-        try:
-            os.chown(path, user_uid, group_gid)
-        except OSError as exc:
-            raise EnvironmentError("chown('%s', %s, %s): %s" %
-                                   (path,
-                                    user_name, group_name,
-                                    exc.strerror))
-
-    # Change permission flags
-    if mode is not None:
-        try:
-            os.chmod(path, mode)
-        except OSError as exc:
-            raise EnvironmentError("chmod('%s', %#o): %s" %
-                                   (path, mode, exc.strerror))
-
-
-def make_dirs(path, mode=None, user=None, group=None, log=None):
-    """Assure directory exists, set ownership and permissions.
-
-    Assure the directory exists and optionally set its ownership
-    and permissions.
-
-    Each of the mode, user and group are optional, if None then
-    that aspect is not modified.
-
-    Owner and group may be specified either with a symbolic name
-    or numeric id.
-
-    :param string path: Pathname of directory whose existence is assured.
-    :param object mode: ownership permissions flags (int) i.e. chmod,
-                        if None do not set.
-    :param object user: set user, name (string) or uid (integer),
-                        if None do not set.
-    :param object group: set group, name (string) or gid (integer)
-                         if None do not set.
-    :param logger log: logging.logger object, used to emit log messages,
-                       if None no logging is performed.
-
-    """
-    if log:
-        if mode is None:
-            mode_string = str(mode)
-        else:
-            mode_string = oct(mode)
-        log.debug("make_dirs path='%s' mode=%s user=%s group=%s",
-                  path, mode_string, user, group)
-
-    if not os.path.exists(path):
-        try:
-            os.makedirs(path)
-        except OSError as exc:
-            raise EnvironmentError("makedirs('%s'): %s" % (path, exc.strerror))
-
-    set_permissions(path, mode, user, group, log)
-
-
 class WhiteListedItemFilter(object):
 
     def __init__(self, whitelist, data):
@@ -498,14 +321,16 @@ def isotime(at=None, subsecond=False):
     Python provides a similar instance method for datetime.datetime objects
     called `isoformat()`. The format of the strings generated by `isoformat()`
     has a couple of problems:
+
     1) The strings generated by `isotime()` are used in tokens and other public
-       APIs that we can't change without a deprecation period. The strings
-       generated by `isoformat()` are not the same format, so we can't just
-       change to it.
+    APIs that we can't change without a deprecation period. The strings
+    generated by `isoformat()` are not the same format, so we can't just
+    change to it.
+
     2) The strings generated by `isoformat()` do not include the microseconds
-       if the value happens to be 0. This will likely show up as random
-       failures as parsers may be written to always expect microseconds, and it
-       will parse correctly most of the time.
+    if the value happens to be 0. This will likely show up as random
+    failures as parsers may be written to always expect microseconds, and it
+    will parse correctly most of the time.
 
     :param at: Optional datetime object to return at a string. If not provided,
                the time when the function was called will be used.
@@ -527,25 +352,21 @@ def isotime(at=None, subsecond=False):
                      if not subsecond
                      else _ISO8601_TIME_FORMAT_SUBSECOND)
     tz = at.tzinfo.tzname(None) if at.tzinfo else 'UTC'
-    st += ('Z' if tz == 'UTC' else tz)
+    # Need to handle either iso8601 or python UTC format
+    st += ('Z' if tz in ['UTC', 'UTC+00:00'] else tz)
     return st
 
 
-def get_token_ref(context):
-    """Retrieve KeystoneToken object from the auth context and returns it.
-
-    :param dict context: The request context.
-    :raises keystone.exception.Unauthorized: If auth context cannot be found.
-    :returns: The KeystoneToken object.
-    """
+def parse_expiration_date(expiration_date):
+    if not expiration_date.endswith('Z'):
+        expiration_date += 'Z'
     try:
-        # Retrieve the auth context that was prepared by AuthContextMiddleware.
-        auth_context = (context['environment']
-                        [authorization.AUTH_CONTEXT_ENV])
-        return auth_context['token']
-    except KeyError:
-        LOG.warning(_LW("Couldn't find the auth context."))
-        raise exception.Unauthorized()
+        expiration_time = timeutils.parse_isotime(expiration_date)
+    except ValueError:
+        raise exception.ValidationTimeStampError()
+    if timeutils.is_older_than(expiration_time, 0):
+        raise exception.ValidationExpirationError()
+    return expiration_time
 
 
 URL_RESERVED_CHARS = ":/?#[]@!$&'()*+,;="
@@ -580,7 +401,7 @@ def remove_standard_port(url):
     # remove the default ports specified in RFC2616 and 2818
     o = moves.urllib.parse.urlparse(url)
     separator = ':'
-    (host, separator, port) = o.netloc.partition(':')
+    (host, separator, port) = o.netloc.partition(separator)
     if o.scheme.lower() == 'http' and port == '80':
         # NOTE(gyee): _replace() is not a private method. It has
         # an underscore prefix to prevent conflict with field names.
@@ -610,25 +431,24 @@ def format_url(url, substitutions, silent_keyerror_failures=None):
     try:
         result = url.replace('$(', '%(') % substitutions
     except AttributeError:
-        msg = _LE("Malformed endpoint - %(url)r is not a string")
+        msg = "Malformed endpoint - %(url)r is not a string"
         LOG.error(msg, {"url": url})
         raise exception.MalformedEndpoint(endpoint=url)
     except KeyError as e:
         if not e.args or e.args[0] not in allow_keyerror:
-            msg = _LE("Malformed endpoint %(url)s - unknown key "
-                      "%(keyerror)s")
+            msg = "Malformed endpoint %(url)s - unknown key %(keyerror)s"
             LOG.error(msg, {"url": url, "keyerror": e})
             raise exception.MalformedEndpoint(endpoint=url)
         else:
             result = None
     except TypeError as e:
-        msg = _LE("Malformed endpoint '%(url)s'. The following type error "
-                  "occurred during string substitution: %(typeerror)s")
+        msg = ("Malformed endpoint '%(url)s'. The following type error "
+               "occurred during string substitution: %(typeerror)s")
         LOG.error(msg, {"url": url, "typeerror": e})
         raise exception.MalformedEndpoint(endpoint=url)
-    except ValueError as e:
-        msg = _LE("Malformed endpoint %s - incomplete format "
-                  "(are you missing a type notifier ?)")
+    except ValueError:
+        msg = ("Malformed endpoint %s - incomplete format "
+               "(are you missing a type notifier ?)")
         LOG.error(msg, url)
         raise exception.MalformedEndpoint(endpoint=url)
     return result
