@@ -18,7 +18,6 @@
 
 """Utility methods for working with WSGI servers."""
 
-import copy
 import functools
 import itertools
 import re
@@ -35,14 +34,13 @@ from six.moves import http_client
 import webob.dec
 import webob.exc
 
-from keystone.common import dependency
+from keystone.common import authorization
 from keystone.common import json_home
 from keystone.common import request as request_mod
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
-from keystone.i18n import _, _LI, _LW
-from keystone.models import token_model
+from keystone.i18n import _
 
 
 CONF = keystone.conf.CONF
@@ -58,65 +56,6 @@ JSON_ENCODE_CONTENT_TYPES = set(['application/json',
                                  'application/json-home'])
 
 
-def validate_token_bind(context, token_ref):
-    bind_mode = CONF.token.enforce_token_bind
-
-    if bind_mode == 'disabled':
-        return
-
-    if not isinstance(token_ref, token_model.KeystoneToken):
-        raise exception.UnexpectedError(_('token reference must be a '
-                                          'KeystoneToken type, got: %s') %
-                                        type(token_ref))
-    bind = token_ref.bind
-
-    # permissive and strict modes don't require there to be a bind
-    permissive = bind_mode in ('permissive', 'strict')
-
-    if not bind:
-        if permissive:
-            # no bind provided and none required
-            return
-        else:
-            LOG.info(_LI("No bind information present in token"))
-            raise exception.Unauthorized()
-
-    # get the named mode if bind_mode is not one of the known
-    name = None if permissive or bind_mode == 'required' else bind_mode
-
-    if name and name not in bind:
-        LOG.info(_LI("Named bind mode %s not in bind information"), name)
-        raise exception.Unauthorized()
-
-    for bind_type, identifier in bind.items():
-        if bind_type == 'kerberos':
-            if (context['environment'].get('AUTH_TYPE', '').lower() !=
-                    'negotiate'):
-                msg = _('Kerberos credentials required and not present')
-                LOG.info(msg)
-                raise exception.Unauthorized(msg)
-
-            if context['environment'].get('REMOTE_USER') != identifier:
-                msg = _('Kerberos credentials do not match those in bind')
-                LOG.info(msg)
-                raise exception.Unauthorized(msg)
-
-            LOG.info(_LI('Kerberos bind authentication successful'))
-
-        elif bind_mode == 'permissive':
-            LOG.debug(("Ignoring unknown bind (due to permissive mode): "
-                       "{%(bind_type)s: %(identifier)s}"), {
-                           'bind_type': bind_type,
-                           'identifier': identifier})
-        else:
-            msg = _('Could not verify unknown bind: {%(bind_type)s: '
-                    '%(identifier)s}') % {
-                        'bind_type': bind_type,
-                        'identifier': identifier}
-            LOG.info(msg)
-            raise exception.Unauthorized(msg)
-
-
 def best_match_language(req):
     """Determine the best available locale.
 
@@ -134,27 +73,7 @@ class BaseApplication(object):
 
     @classmethod
     def factory(cls, global_config, **local_config):
-        """Used for paste app factories in paste.deploy config files.
-
-        Any local configuration (that is, values under the [app:APPNAME]
-        section of the paste config) will be passed into the `__init__` method
-        as kwargs.
-
-        A hypothetical configuration would look like:
-
-            [app:wadl]
-            latest_version = 1.3
-            paste.app_factory = keystone.fancy_api:Wadl.factory
-
-        which would result in a call to the `Wadl` class as
-
-            import keystone.fancy_api
-            keystone.fancy_api.Wadl(latest_version='1.3')
-
-        You could of course re-implement the `factory` method in subclasses,
-        but using the kwarg passing it shouldn't be necessary.
-
-        """
+        """Used for loading in middleware (holdover from paste.deploy)."""
         return cls(**local_config)
 
     def __call__(self, environ, start_response):
@@ -192,11 +111,15 @@ class BaseApplication(object):
         See the end of http://pythonpaste.org/webob/modules/dec.html
         for more info.
 
+        NOTE: this is now strictly used in conversion from old wsgi
+        implementation to flask. Once the flask implementation is complete,
+        the __call__ will not be needed as the flask app will handle
+        dispatching and __call__.
+
         """
         raise NotImplementedError('You must implement __call__')
 
 
-@dependency.requires('assignment_api', 'policy_api', 'token_provider_api')
 class Application(BaseApplication):
 
     @webob.dec.wsgify(RequestClass=request_mod.Request)
@@ -225,14 +148,17 @@ class Application(BaseApplication):
             result = method(req, **params)
         except exception.Unauthorized as e:
             LOG.warning(
-                _LW("Authorization failed. %(exception)s from "
-                    "%(remote_addr)s"),
+                "Authorization failed. %(exception)s from "
+                "%(remote_addr)s",
                 {'exception': e, 'remote_addr': req.environ['REMOTE_ADDR']})
             return render_exception(e,
                                     context=req.context_dict,
                                     user_locale=best_match_language(req))
         except exception.Error as e:
-            LOG.warning(six.text_type(e))
+            if isinstance(e, exception.UnexpectedError):
+                LOG.exception(six.text_type(e))
+            else:
+                LOG.warning(six.text_type(e))
             return render_exception(e,
                                     context=req.context_dict,
                                     user_locale=best_match_language(req))
@@ -288,28 +214,7 @@ class Application(BaseApplication):
             does not have the admin role
 
         """
-        request.assert_authenticated()
-
-        if not request.context.is_admin:
-            user_token_ref = utils.get_token_ref(request.context_dict)
-
-            creds = copy.deepcopy(user_token_ref.metadata)
-
-            try:
-                creds['user_id'] = user_token_ref.user_id
-            except exception.UnexpectedError:
-                LOG.debug('Invalid user')
-                raise exception.Unauthorized()
-
-            if user_token_ref.project_scoped:
-                creds['tenant_id'] = user_token_ref.project_id
-            else:
-                LOG.debug('Invalid tenant')
-                raise exception.Unauthorized()
-
-            creds['roles'] = user_token_ref.role_names
-            # Accept either is_admin or the admin role
-            self.policy_api.enforce(creds, 'admin_required', {})
+        authorization.assert_admin(self, request)
 
     def _attribute_is_empty(self, ref, attribute):
         """Determine if the attribute in ref is empty or None."""
@@ -337,8 +242,8 @@ class Application(BaseApplication):
             raise exception.ValidationError(message=msg)
 
     @classmethod
-    def base_url(cls, context, endpoint_type):
-        url = CONF['%s_endpoint' % endpoint_type]
+    def base_url(cls, context, endpoint_type=None):
+        url = CONF['public_endpoint']
 
         if url:
             substitutions = dict(
@@ -403,7 +308,7 @@ class Middleware(Application):
             return cls(app)
         return _factory
 
-    def __init__(self, application):
+    def __init__(self, application, conf=None):
         super(Middleware, self).__init__()
         self.application = application
 
@@ -524,7 +429,8 @@ class Router(object):
         """
         match = req.environ['wsgiorg.routing_args'][1]
         if not match:
-            msg = _('The resource could not be found.')
+            msg = (_('(%(url)s): The resource could not be found.') %
+                   {'url': req.url})
             return render_exception(exception.NotFound(msg),
                                     request=req,
                                     user_locale=best_match_language(req))
@@ -576,27 +482,7 @@ class ExtensionRouter(Router):
 
     @classmethod
     def factory(cls, global_config, **local_config):
-        """Used for paste app factories in paste.deploy config files.
-
-        Any local configuration (that is, values under the [filter:APPNAME]
-        section of the paste config) will be passed into the `__init__` method
-        as kwargs.
-
-        A hypothetical configuration would look like:
-
-            [filter:analytics]
-            redis_host = 127.0.0.1
-            paste.filter_factory = keystone.analytics:Analytics.factory
-
-        which would result in a call to the `Analytics` class as
-
-            import keystone.analytics
-            keystone.analytics.Analytics(app, redis_host='127.0.0.1')
-
-        You could of course re-implement the `factory` method in subclasses,
-        but using the kwarg passing it shouldn't be necessary.
-
-        """
+        """Used for loading in middleware (holdover from paste.deploy)."""
         def _factory(app):
             conf = global_config.copy()
             conf.update(local_config)
@@ -668,6 +554,7 @@ class RoutersBase(object):
         json_home.Status.update_resource_data(resource_data, status)
 
         self.v3_resources.append((rel, resource_data))
+        json_home.JsonHomeResources.append_resource(rel, resource_data)
 
 
 class V3ExtensionRouter(ExtensionRouter, RoutersBase):
@@ -762,7 +649,8 @@ def render_response(body=None, status=None, headers=None, method=None):
 
     resp = webob.Response(body=body,
                           status='%d %s' % status,
-                          headerlist=headers)
+                          headerlist=headers,
+                          charset='utf-8')
 
     if method and method.upper() == 'HEAD':
         # NOTE(morganfainberg): HEAD requests should return the same status
@@ -807,7 +695,7 @@ def render_exception(error, context=None, request=None, user_locale=None):
             local_context = {'environment': request.environ}
         elif context and 'environment' in context:
             local_context = {'environment': context['environment']}
-        url = Application.base_url(local_context, 'public')
+        url = Application.base_url(local_context)
 
         headers.append(('WWW-Authenticate', 'Keystone uri="%s"' % url))
     return render_response(status=(error.code, error.title),
