@@ -42,20 +42,20 @@ import six
 from six.moves import http_client
 
 from keystone.common import controller
-from keystone.common import dependency
+from keystone.common import provider_api
 from keystone.common import utils
 from keystone.common import wsgi
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 
 CRED_TYPE_EC2 = 'ec2'
+CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
-@dependency.requires('assignment_api', 'catalog_api', 'credential_api',
-                     'identity_api', 'resource_api', 'role_api',
-                     'token_provider_api')
 @six.add_metaclass(abc.ABCMeta)
-class Ec2ControllerCommon(object):
+class Ec2ControllerCommon(provider_api.ProviderAPIMixin, object):
     def check_signature(self, creds_ref, credentials):
         signer = ec2_utils.Ec2Signer(creds_ref['secret'])
         signature = signer.generate(credentials)
@@ -113,7 +113,7 @@ class Ec2ControllerCommon(object):
     def _authenticate(self, credentials=None, ec2credentials=None):
         """Common code shared between the V2 and V3 authenticate methods.
 
-        :returns: user_ref, tenant_ref, metadata_ref, roles_ref, catalog_ref
+        :returns: user_ref, tenant_ref, roles_ref
         """
         # FIXME(ja): validate that a service token was used!
 
@@ -132,15 +132,6 @@ class Ec2ControllerCommon(object):
         # TODO(termie): this is copied from TokenController.authenticate
         tenant_ref = self.resource_api.get_project(creds_ref['tenant_id'])
         user_ref = self.identity_api.get_user(creds_ref['user_id'])
-        metadata_ref = {}
-        metadata_ref['roles'] = (
-            self.assignment_api.get_roles_for_user_and_project(
-                user_ref['id'], tenant_ref['id']))
-
-        trust_id = creds_ref.get('trust_id')
-        if trust_id:
-            metadata_ref['trust_id'] = trust_id
-            metadata_ref['trustee_user_id'] = user_ref['id']
 
         # Validate that the auth info is valid and nothing is disabled
         try:
@@ -154,16 +145,15 @@ class Ec2ControllerCommon(object):
             six.reraise(exception.Unauthorized, exception.Unauthorized(e),
                         sys.exc_info()[2])
 
-        roles = metadata_ref.get('roles', [])
+        roles = self.assignment_api.get_roles_for_user_and_project(
+            user_ref['id'], tenant_ref['id']
+        )
         if not roles:
             raise exception.Unauthorized(
                 message=_('User not valid for tenant.'))
         roles_ref = [self.role_api.get_role(role_id) for role_id in roles]
 
-        catalog_ref = self.catalog_api.get_catalog(
-            user_ref['id'], tenant_ref['id'])
-
-        return user_ref, tenant_ref, metadata_ref, roles_ref, catalog_ref
+        return user_ref, tenant_ref, roles_ref
 
     def create_credential(self, request, user_id, tenant_id):
         """Create a secret/access pair for use with ec2 style auth.
@@ -260,101 +250,21 @@ class Ec2ControllerCommon(object):
                 message=_('EC2 access key not found.'))
         return self._convert_v3_to_ec2_credential(cred)
 
+    def render_token_data_response(self, token_id, token_data):
+        """Render token data HTTP response.
 
-@dependency.requires('policy_api', 'token_provider_api')
-class Ec2Controller(Ec2ControllerCommon, controller.V2Controller):
-
-    @controller.v2_ec2_deprecated
-    def authenticate(self, request, credentials=None, ec2Credentials=None):
-        (user_ref, tenant_ref, metadata_ref, roles_ref,
-         catalog_ref) = self._authenticate(credentials=credentials,
-                                           ec2credentials=ec2Credentials)
-
-        # NOTE(morganfainberg): Make sure the data is in correct form since it
-        # might be consumed external to Keystone and this is a v2.0 controller.
-        # The token provider does not explicitly care about user_ref version
-        # in this case, but the data is stored in the token itself and should
-        # match the version
-        user_ref = self.v3_to_v2_user(user_ref)
-        auth_token_data = dict(user=user_ref,
-                               tenant=tenant_ref,
-                               metadata=metadata_ref,
-                               id='placeholder')
-        (token_id, token_data) = self.token_provider_api.issue_v2_token(
-            auth_token_data, roles_ref, catalog_ref)
-        return token_data
-
-    @controller.v2_ec2_deprecated
-    def get_credential(self, request, user_id, credential_id):
-        if not self._is_admin(request):
-            self._assert_identity(request.context_dict, user_id)
-        return super(Ec2Controller, self).get_credential(user_id,
-                                                         credential_id)
-
-    @controller.v2_ec2_deprecated
-    def get_credentials(self, request, user_id):
-        if not self._is_admin(request):
-            self._assert_identity(request.context_dict, user_id)
-        return super(Ec2Controller, self).get_credentials(user_id)
-
-    @controller.v2_ec2_deprecated
-    def create_credential(self, request, user_id, tenant_id):
-        if not self._is_admin(request):
-            self._assert_identity(request.context_dict, user_id)
-        return super(Ec2Controller, self).create_credential(
-            request, user_id, tenant_id)
-
-    @controller.v2_ec2_deprecated
-    def delete_credential(self, request, user_id, credential_id):
-        if not self._is_admin(request):
-            self._assert_identity(request.context_dict, user_id)
-            self._assert_owner(user_id, credential_id)
-        return super(Ec2Controller, self).delete_credential(user_id,
-                                                            credential_id)
-
-    def _assert_identity(self, context, user_id):
-        """Check that the provided token belongs to the user.
-
-        :param context: standard context
-        :param user_id: id of user
-        :raises keystone.exception.Forbidden: when token is invalid
+        Stash token ID into the X-Subject-Token header.
 
         """
-        token_ref = utils.get_token_ref(context)
+        status = (http_client.OK,
+                  http_client.responses[http_client.OK])
+        headers = [('X-Subject-Token', token_id)]
 
-        if token_ref.user_id != user_id:
-            raise exception.Forbidden(_('Token belongs to another user'))
-
-    def _is_admin(self, request):
-        """Wrap admin assertion error return statement.
-
-        :param context: standard context
-        :returns: bool: success
-
-        """
-        try:
-            # NOTE(morganfainberg): policy_api is required for assert_admin
-            # to properly perform policy enforcement.
-            self.assert_admin(request)
-            return True
-        except (exception.Forbidden, exception.Unauthorized):
-            return False
-
-    def _assert_owner(self, user_id, credential_id):
-        """Ensure the provided user owns the credential.
-
-        :param user_id: expected credential owner
-        :param credential_id: id of credential object
-        :raises keystone.exception.Forbidden: on failure
-
-        """
-        ec2_credential_id = utils.hash_access_key(credential_id)
-        cred_ref = self.credential_api.get_credential(ec2_credential_id)
-        if user_id != cred_ref['user_id']:
-            raise exception.Forbidden(_('Credential belongs to another user'))
+        return wsgi.render_response(body=token_data,
+                                    status=status,
+                                    headers=headers)
 
 
-@dependency.requires('policy_api', 'token_provider_api')
 class Ec2ControllerV3(Ec2ControllerCommon, controller.V3Controller):
 
     collection_name = 'credentials'
@@ -377,16 +287,17 @@ class Ec2ControllerV3(Ec2ControllerCommon, controller.V3Controller):
         self.check_protection(request, prep_info, ref)
 
     def authenticate(self, context, credentials=None, ec2Credentials=None):
-        (user_ref, project_ref, metadata_ref, roles_ref,
-         catalog_ref) = self._authenticate(credentials=credentials,
-                                           ec2credentials=ec2Credentials)
+        (user_ref, project_ref, roles_ref) = self._authenticate(
+            credentials=credentials, ec2credentials=ec2Credentials
+        )
 
         method_names = ['ec2credential']
 
-        token_id, token_data = self.token_provider_api.issue_v3_token(
-            user_ref['id'], method_names, project_id=project_ref['id'],
-            metadata_ref=metadata_ref)
-        return render_token_data_response(token_id, token_data)
+        token = self.token_provider_api.issue_token(
+            user_ref['id'], method_names, project_id=project_ref['id']
+        )
+        token_reference = controller.render_token_response_from_model(token)
+        return self.render_token_data_response(token.id, token_reference)
 
     @controller.protected(callback=_check_credential_owner_and_user_id_match)
     def ec2_get_credential(self, request, user_id, credential_id):
@@ -421,17 +332,3 @@ class Ec2ControllerV3(Ec2ControllerCommon, controller.V3Controller):
             'credential_id': ref['access']}
         ref.setdefault('links', {})
         ref['links']['self'] = url
-
-
-def render_token_data_response(token_id, token_data):
-    """Render token data HTTP response.
-
-    Stash token ID into the X-Subject-Token header.
-
-    """
-    headers = [('X-Subject-Token', token_id)]
-
-    return wsgi.render_response(body=token_data,
-                                status=(http_client.OK,
-                                        http_client.responses[http_client.OK]),
-                                headers=headers)
