@@ -20,7 +20,7 @@ from keystone.common import fernet_utils
 import keystone.conf
 from keystone.credential.providers import core
 from keystone import exception
-from keystone.i18n import _, _LW
+from keystone.i18n import _
 
 
 CONF = keystone.conf.CONF
@@ -43,7 +43,8 @@ MAX_ACTIVE_KEYS = 3
 
 def get_multi_fernet_keys():
     key_utils = fernet_utils.FernetUtils(
-        CONF.credential.key_repository, MAX_ACTIVE_KEYS)
+        CONF.credential.key_repository, MAX_ACTIVE_KEYS,
+        'credential')
     keys = key_utils.load_keys(use_null_key=True)
 
     fernet_keys = [fernet.Fernet(key) for key in keys]
@@ -56,7 +57,11 @@ def primary_key_hash(keys):
     """Calculate a hash of the primary key used for encryption."""
     if isinstance(keys[0], six.text_type):
         keys[0] = keys[0].encode('utf-8')
-    return hashlib.sha1(keys[0]).hexdigest()
+    # NOTE(lhinds) This is marked as #nosec since bandit will see SHA1 which
+    # is marked as insecure. However, this hash function is used alongside
+    # encrypted blobs to implement HMAC-SHA1, which is currently not insecure
+    # but will still trigger when scanned by bandit.
+    return hashlib.sha1(keys[0]).hexdigest()  # nosec
 
 
 class Provider(core.Provider):
@@ -69,18 +74,18 @@ class Provider(core.Provider):
         crypto, keys = get_multi_fernet_keys()
 
         if keys[0] == fernet_utils.NULL_KEY:
-            LOG.warning(_LW(
+            LOG.warning(
                 'Encrypting credentials with the null key. Please properly '
                 'encrypt credentials using `keystone-manage credential_setup`,'
                 ' `keystone-manage credential_migrate`, and `keystone-manage '
-                'credential_rotate`'))
+                'credential_rotate`')
 
         try:
             return (
                 crypto.encrypt(credential.encode('utf-8')),
                 primary_key_hash(keys))
         except (TypeError, ValueError) as e:
-            msg = 'Credential could not be encrypted: %s' % str(e)
+            msg = _('Credential could not be encrypted: %s') % str(e)
             LOG.error(msg)
             raise exception.CredentialEncryptionError(msg)
 
