@@ -15,9 +15,10 @@
 from oslo_log import log
 from oslo_utils import encodeutils
 import six
+from six.moves import http_client
 
 import keystone.conf
-from keystone.i18n import _, _LW
+from keystone.i18n import _
 
 
 CONF = keystone.conf.CONF
@@ -63,7 +64,7 @@ class Error(Exception):
             if _FATAL_EXCEPTION_FORMAT_ERRORS:
                 raise
             else:
-                LOG.warning(_LW('missing exception kwargs (programmer error)'))
+                LOG.warning('missing exception kwargs (programmer error)')
                 message = self.message_format
 
         super(Error, self).__init__(message)
@@ -80,21 +81,33 @@ class Error(Exception):
 
 
 class ValidationError(Error):
-    message_format = _("Expecting to find %(attribute)s in %(target)s -"
-                       " the server could not comply with the request"
+    message_format = _("Expecting to find %(attribute)s in %(target)s."
+                       " The server could not comply with the request"
                        " since it is either malformed or otherwise"
                        " incorrect. The client is assumed to be in error.")
-    code = 400
-    title = 'Bad Request'
+    code = int(http_client.BAD_REQUEST)
+    title = http_client.responses[http_client.BAD_REQUEST]
 
 
 class URLValidationError(ValidationError):
     message_format = _("Cannot create an endpoint with an invalid URL:"
-                       " %(url)s")
+                       " %(url)s.")
 
 
 class PasswordValidationError(ValidationError):
-    message_format = _("Password validation error: %(detail)s")
+    message_format = _("Password validation error: %(detail)s.")
+
+
+class PasswordRequirementsValidationError(PasswordValidationError):
+    message_format = _("The password does not match the requirements:"
+                       " %(detail)s.")
+
+
+class PasswordHistoryValidationError(PasswordValidationError):
+    message_format = _("The new password cannot be identical to a "
+                       "previous password. The total number which"
+                       "includes the new password must be unique is "
+                       "%(unique_count)s.")
 
 
 class PasswordAgeValidationError(PasswordValidationError):
@@ -106,6 +119,13 @@ class PasswordAgeValidationError(PasswordValidationError):
                        "reset your password.")
 
 
+class PasswordSelfServiceDisabled(PasswordValidationError):
+    message_format = _("You cannot change your password at this time due "
+                       "to password policy disallowing password changes. "
+                       "Please contact your administrator to reset your "
+                       "password.")
+
+
 class SchemaValidationError(ValidationError):
     # NOTE(lbragstad): For whole OpenStack message consistency, this error
     # message has been written in a format consistent with WSME.
@@ -117,8 +137,14 @@ class ValidationTimeStampError(Error):
                        " The server could not comply with the request"
                        " since it is either malformed or otherwise"
                        " incorrect. The client is assumed to be in error.")
-    code = 400
-    title = 'Bad Request'
+    code = int(http_client.BAD_REQUEST)
+    title = http_client.responses[http_client.BAD_REQUEST]
+
+
+class InvalidOperatorError(ValidationError):
+    message_format = _("The given operator %(_op)s is not valid."
+                       " It must be one of the following:"
+                       " 'eq', 'neq', 'lt', 'lte', 'gt', or 'gte'.")
 
 
 class ValidationExpirationError(Error):
@@ -126,31 +152,31 @@ class ValidationExpirationError(Error):
                        " The server could not comply with the request"
                        " since it is either malformed or otherwise"
                        " incorrect. The client is assumed to be in error.")
-    code = 400
-    title = 'Bad Request'
+    code = int(http_client.BAD_REQUEST)
+    title = http_client.responses[http_client.BAD_REQUEST]
 
 
 class StringLengthExceeded(ValidationError):
-    message_format = _("String length exceeded.The length of"
-                       " string '%(string)s' exceeded the limit"
+    message_format = _("String length exceeded. The length of"
+                       " string '%(string)s' exceeds the limit"
                        " of column %(type)s(CHAR(%(length)d)).")
 
 
-class ValidationSizeError(Error):
-    message_format = _("Request attribute %(attribute)s must be"
-                       " less than or equal to %(size)i. The server"
-                       " could not comply with the request because"
-                       " the attribute size is invalid (too large)."
-                       " The client is assumed to be in error.")
-    code = 400
-    title = 'Bad Request'
+class AmbiguityError(ValidationError):
+    message_format = _("There are multiple %(resource)s entities named"
+                       " '%(name)s'. Please use ID instead of names to"
+                       " resolve the ambiguity.")
+
+
+class ApplicationCredentialValidationError(ValidationError):
+    message_format = _("Invalid application credential: %(detail)s")
 
 
 class CircularRegionHierarchyError(Error):
     message_format = _("The specified parent region %(parent_region_id)s "
                        "would create a circular region hierarchy.")
-    code = 400
-    title = 'Bad Request'
+    code = int(http_client.BAD_REQUEST)
+    title = http_client.responses[http_client.BAD_REQUEST]
 
 
 class ForbiddenNotSecurity(Error):
@@ -161,8 +187,8 @@ class ForbiddenNotSecurity(Error):
 
     """
 
-    code = 403
-    title = 'Forbidden'
+    code = int(http_client.FORBIDDEN)
+    title = http_client.responses[http_client.FORBIDDEN]
 
 
 class PasswordVerificationError(ForbiddenNotSecurity):
@@ -176,10 +202,9 @@ class RegionDeletionError(ForbiddenNotSecurity):
                        "its child regions have associated endpoints.")
 
 
-class PKITokenExpected(ForbiddenNotSecurity):
-    message_format = _('The certificates you requested are not available. '
-                       'It is likely that this server does not use PKI tokens '
-                       'otherwise this is the result of misconfiguration.')
+class ApplicationCredentialLimitExceeded(ForbiddenNotSecurity):
+    message_format = _("Unable to create additional application credentials, "
+                       "maximum of %(limit)d already exceeded for user.")
 
 
 class SecurityError(Error):
@@ -237,13 +262,22 @@ class SecurityError(Error):
 
 class Unauthorized(SecurityError):
     message_format = _("The request you have made requires authentication.")
+    code = int(http_client.UNAUTHORIZED)
+    title = http_client.responses[http_client.UNAUTHORIZED]
+
+
+class InsufficientAuthMethods(Error):
+    # NOTE(notmorgan): This is not a security error, this is meant to
+    # communicate real information back to the user.
+    message_format = _("Insufficient auth methods received for %(user_id)s. "
+                       "Auth Methods Provided: %(methods)s.")
     code = 401
     title = 'Unauthorized'
 
 
 class PasswordExpired(Unauthorized):
-    message_format = _("The password is expired and needs to be reset by an "
-                       "administrator for user: %(user_id)s")
+    message_format = _("The password is expired and needs to be changed for "
+                       "user: %(user_id)s.")
 
 
 class AuthPluginException(Unauthorized):
@@ -255,11 +289,11 @@ class AuthPluginException(Unauthorized):
 
 
 class UserDisabled(Unauthorized):
-    message_format = _("The account is disabled for user: %(user_id)s")
+    message_format = _("The account is disabled for user: %(user_id)s.")
 
 
 class AccountLocked(Unauthorized):
-    message_format = _("The account is locked for user: %(user_id)s")
+    message_format = _("The account is locked for user: %(user_id)s.")
 
 
 class AuthMethodNotSupported(AuthPluginException):
@@ -270,6 +304,11 @@ class AuthMethodNotSupported(AuthPluginException):
         self.authentication = {'methods': CONF.auth.methods}
 
 
+class ApplicationCredentialAuthError(AuthPluginException):
+    message_format = _(
+        "Error authenticating with application credential: %(detail)s")
+
+
 class AdditionalAuthRequired(AuthPluginException):
     message_format = _("Additional authentications steps required.")
 
@@ -281,153 +320,159 @@ class AdditionalAuthRequired(AuthPluginException):
 class Forbidden(SecurityError):
     message_format = _("You are not authorized to perform the"
                        " requested action.")
-    code = 403
-    title = 'Forbidden'
+    code = int(http_client.FORBIDDEN)
+    title = http_client.responses[http_client.FORBIDDEN]
 
 
 class ForbiddenAction(Forbidden):
     message_format = _("You are not authorized to perform the"
-                       " requested action: %(action)s")
-
-
-class ImmutableAttributeError(Forbidden):
-    message_format = _("Could not change immutable attribute(s) "
-                       "'%(attributes)s' in target %(target)s")
+                       " requested action: %(action)s.")
 
 
 class CrossBackendNotAllowed(Forbidden):
     message_format = _("Group membership across backend boundaries is not "
-                       "allowed, group in question is %(group_id)s, "
-                       "user is %(user_id)s")
+                       "allowed. Group in question is %(group_id)s, "
+                       "user is %(user_id)s.")
 
 
 class InvalidPolicyAssociation(Forbidden):
-    message_format = _("Invalid mix of entities for policy association - "
-                       "only Endpoint, Service or Region+Service allowed. "
+    message_format = _("Invalid mix of entities for policy association: "
+                       "only Endpoint, Service, or Region+Service allowed. "
                        "Request was - Endpoint: %(endpoint_id)s, "
-                       "Service: %(service_id)s, Region: %(region_id)s")
+                       "Service: %(service_id)s, Region: %(region_id)s.")
 
 
 class InvalidDomainConfig(Forbidden):
-    message_format = _("Invalid domain specific configuration: %(reason)s")
+    message_format = _("Invalid domain specific configuration: %(reason)s.")
+
+
+class InvalidLimit(Forbidden):
+    message_format = _("Invalid resource limit: %(reason)s.")
+
+
+class LimitTreeExceedError(Exception):
+    def __init__(self, project_id, max_limit_depth):
+        super(LimitTreeExceedError, self).__init__(_(
+            "Keystone cannot start due to project hierarchical depth in the "
+            "current deployment (project_ids: %(project_id)s) exceeds the "
+            "enforcement model's maximum limit of %(max_limit_depth)s. Please "
+            "use a different enforcement model to correct the issue."
+        ) % {'project_id': project_id, 'max_limit_depth': max_limit_depth})
 
 
 class NotFound(Error):
-    message_format = _("Could not find: %(target)s")
-    code = 404
-    title = 'Not Found'
+    message_format = _("Could not find: %(target)s.")
+    code = int(http_client.NOT_FOUND)
+    title = http_client.responses[http_client.NOT_FOUND]
 
 
 class EndpointNotFound(NotFound):
-    message_format = _("Could not find endpoint: %(endpoint_id)s")
-
-
-class MetadataNotFound(NotFound):
-    # NOTE (dolph): metadata is not a user-facing concept,
-    # so this exception should not be exposed.
-
-    message_format = _("An unhandled exception has occurred:"
-                       " Could not find metadata.")
+    message_format = _("Could not find endpoint: %(endpoint_id)s.")
 
 
 class PolicyNotFound(NotFound):
-    message_format = _("Could not find policy: %(policy_id)s")
+    message_format = _("Could not find policy: %(policy_id)s.")
 
 
 class PolicyAssociationNotFound(NotFound):
-    message_format = _("Could not find policy association")
+    message_format = _("Could not find policy association.")
 
 
 class RoleNotFound(NotFound):
-    message_format = _("Could not find role: %(role_id)s")
+    message_format = _("Could not find role: %(role_id)s.")
 
 
 class ImpliedRoleNotFound(NotFound):
-    message_format = _("%(prior_role_id)s does not imply %(implied_role_id)s")
+    message_format = _("%(prior_role_id)s does not imply %(implied_role_id)s.")
 
 
 class InvalidImpliedRole(Forbidden):
-    message_format = _("%(role_id)s cannot be an implied roles")
+    message_format = _("%(role_id)s cannot be an implied roles.")
 
 
 class DomainSpecificRoleMismatch(Forbidden):
-    message_format = _("project: %(project_id)s must be in the same domain "
-                       " as the role: %(role_id)s being assigned.")
+    message_format = _("Project %(project_id)s must be in the same domain "
+                       "as the role %(role_id)s being assigned.")
+
+
+class DomainSpecificRoleNotWithinIdPDomain(Forbidden):
+    message_format = _("role: %(role_name)s must be within the same domain as "
+                       "the identity provider: %(identity_provider)s.")
 
 
 class RoleAssignmentNotFound(NotFound):
     message_format = _("Could not find role assignment with role: "
                        "%(role_id)s, user or group: %(actor_id)s, "
-                       "project or domain: %(target_id)s")
+                       "project, domain, or system: %(target_id)s.")
 
 
 class RegionNotFound(NotFound):
-    message_format = _("Could not find region: %(region_id)s")
+    message_format = _("Could not find region: %(region_id)s.")
 
 
 class ServiceNotFound(NotFound):
-    message_format = _("Could not find service: %(service_id)s")
+    message_format = _("Could not find service: %(service_id)s.")
 
 
 class DomainNotFound(NotFound):
-    message_format = _("Could not find domain: %(domain_id)s")
+    message_format = _("Could not find domain: %(domain_id)s.")
 
 
 class ProjectNotFound(NotFound):
-    message_format = _("Could not find project: %(project_id)s")
+    message_format = _("Could not find project: %(project_id)s.")
 
 
-class InvalidParentProject(NotFound):
-    message_format = _("Cannot create project with parent: %(project_id)s")
+class ProjectTagNotFound(NotFound):
+    message_format = _("Could not find project tag: %(project_tag)s.")
 
 
 class TokenNotFound(NotFound):
-    message_format = _("Could not find token: %(token_id)s")
+    message_format = _("Could not find token: %(token_id)s.")
 
 
 class UserNotFound(NotFound):
-    message_format = _("Could not find user: %(user_id)s")
+    message_format = _("Could not find user: %(user_id)s.")
 
 
 class GroupNotFound(NotFound):
-    message_format = _("Could not find group: %(group_id)s")
+    message_format = _("Could not find group: %(group_id)s.")
 
 
 class MappingNotFound(NotFound):
-    message_format = _("Could not find mapping: %(mapping_id)s")
+    message_format = _("Could not find mapping: %(mapping_id)s.")
 
 
 class TrustNotFound(NotFound):
-    message_format = _("Could not find trust: %(trust_id)s")
+    message_format = _("Could not find trust: %(trust_id)s.")
 
 
 class TrustUseLimitReached(Forbidden):
-    message_format = _("No remaining uses for trust: %(trust_id)s")
+    message_format = _("No remaining uses for trust: %(trust_id)s.")
 
 
 class CredentialNotFound(NotFound):
-    message_format = _("Could not find credential: %(credential_id)s")
+    message_format = _("Could not find credential: %(credential_id)s.")
 
 
 class VersionNotFound(NotFound):
-    message_format = _("Could not find version: %(version)s")
+    message_format = _("Could not find version: %(version)s.")
 
 
 class EndpointGroupNotFound(NotFound):
-    message_format = _("Could not find Endpoint Group: %(endpoint_group_id)s")
+    message_format = _("Could not find Endpoint Group: %(endpoint_group_id)s.")
 
 
 class IdentityProviderNotFound(NotFound):
-    message_format = _("Could not find Identity Provider: %(idp_id)s")
+    message_format = _("Could not find Identity Provider: %(idp_id)s.")
 
 
 class ServiceProviderNotFound(NotFound):
-    message_format = _("Could not find Service Provider: %(sp_id)s")
+    message_format = _("Could not find Service Provider: %(sp_id)s.")
 
 
 class FederatedProtocolNotFound(NotFound):
     message_format = _("Could not find federated protocol %(protocol_id)s for"
-                       " Identity Provider: %(idp_id)s")
+                       " Identity Provider: %(idp_id)s.")
 
 
 class PublicIDNotFound(NotFound):
@@ -436,9 +481,27 @@ class PublicIDNotFound(NotFound):
     message_format = "%(id)s"
 
 
+class RegisteredLimitNotFound(NotFound):
+    message_format = _("Could not find registered limit for %(id)s.")
+
+
+class LimitNotFound(NotFound):
+    message_format = _("Could not find limit for %(id)s.")
+
+
+class NoLimitReference(Forbidden):
+    message_format = _("Unable to create a limit that has no corresponding "
+                       "registered limit.")
+
+
+class RegisteredLimitError(ForbiddenNotSecurity):
+    message_format = _("Unable to update or delete registered limit %(id)s "
+                       "because there are project limits associated with it.")
+
+
 class DomainConfigNotFound(NotFound):
     message_format = _('Could not find %(group_or_option)s in domain '
-                       'configuration for domain %(domain_id)s')
+                       'configuration for domain %(domain_id)s.')
 
 
 class ConfigRegistrationNotFound(Exception):
@@ -448,17 +511,16 @@ class ConfigRegistrationNotFound(Exception):
     pass
 
 
-class KeystoneConfigurationError(Exception):
-    # This is an exception to be used in the case that Keystone config is
-    # invalid and Keystone should not start.
-    pass
+class ApplicationCredentialNotFound(NotFound):
+    message_format = _("Could not find Application Credential: "
+                       "%(application_credential_id)s.")
 
 
 class Conflict(Error):
     message_format = _("Conflict occurred attempting to store %(type)s -"
-                       " %(details)s")
-    code = 409
-    title = 'Conflict'
+                       " %(details)s.")
+    code = int(http_client.CONFLICT)
+    title = http_client.responses[http_client.CONFLICT]
 
 
 class UnexpectedError(SecurityError):
@@ -468,7 +530,7 @@ class UnexpectedError(SecurityError):
                        "from fulfilling your request.")
 
     debug_message_format = _("An unexpected error prevented the server "
-                             "from fulfilling your request: %(exception)s")
+                             "from fulfilling your request: %(exception)s.")
 
     def _build_message(self, message, **kwargs):
 
@@ -480,12 +542,12 @@ class UnexpectedError(SecurityError):
         return super(UnexpectedError, self)._build_message(
             message or self.debug_message_format, **kwargs)
 
-    code = 500
-    title = 'Internal Server Error'
+    code = int(http_client.INTERNAL_SERVER_ERROR)
+    title = http_client.responses[http_client.INTERNAL_SERVER_ERROR]
 
 
 class TrustConsumeMaximumAttempt(UnexpectedError):
-    debug_message_format = _("Unable to consume trust %(trust_id)s, unable to "
+    debug_message_format = _("Unable to consume trust %(trust_id)s. Unable to "
                              "acquire lock.")
 
 
@@ -506,34 +568,34 @@ class MappedGroupNotFound(UnexpectedError):
 
 
 class MetadataFileError(UnexpectedError):
-    debug_message_format = _("Error while reading metadata file, %(reason)s")
+    debug_message_format = _("Error while reading metadata file: %(reason)s.")
 
 
 class DirectMappingError(UnexpectedError):
-    message_format = _("Local section in mapping %(mapping_id)s refers to a "
-                       "remote match that doesn't exist "
-                       "(e.g. {0} in a local section).")
+    debug_message_format = _("Local section in mapping %(mapping_id)s refers "
+                             "to a remote match that doesn't exist "
+                             "(e.g. {0} in a local section).")
 
 
 class AssignmentTypeCalculationError(UnexpectedError):
     debug_message_format = _(
         'Unexpected combination of grant attributes - '
         'User: %(user_id)s, Group: %(group_id)s, Project: %(project_id)s, '
-        'Domain: %(domain_id)s')
+        'Domain: %(domain_id)s.')
 
 
 class NotImplemented(Error):
     message_format = _("The action you have requested has not"
                        " been implemented.")
-    code = 501
-    title = 'Not Implemented'
+    code = int(http_client.NOT_IMPLEMENTED)
+    title = http_client.responses[http_client.NOT_IMPLEMENTED]
 
 
 class Gone(Error):
     message_format = _("The service you have requested is no"
                        " longer available on this server.")
-    code = 410
-    title = 'Gone'
+    code = int(http_client.GONE)
+    title = http_client.responses[http_client.GONE]
 
 
 class ConfigFileNotFound(UnexpectedError):
@@ -568,13 +630,13 @@ class UnsupportedTokenVersionException(UnexpectedError):
 class SAMLSigningError(UnexpectedError):
     debug_message_format = _('Unable to sign SAML assertion. It is likely '
                              'that this server does not have xmlsec1 '
-                             'installed, or this is the result of '
-                             'misconfiguration. Reason %(reason)s')
+                             'installed or this is the result of '
+                             'misconfiguration. Reason %(reason)s.')
 
 
 class OAuthHeadersMissingError(UnexpectedError):
     debug_message_format = _('No Authorization headers found, cannot proceed '
-                             'with OAuth related calls, if running under '
+                             'with OAuth related calls. If running under '
                              'HTTPd or Apache, ensure WSGIPassAuthorization '
                              'is set to On.')
 
@@ -585,10 +647,32 @@ class TokenlessAuthConfigError(ValidationError):
                        'was not found in the request environment.')
 
 
-class UnsupportedDriverVersion(UnexpectedError):
-    debug_message_format = _('%(driver)s is not supported driver version')
-
-
 class CredentialEncryptionError(Exception):
     message_format = _("An unexpected error prevented the server "
                        "from accessing encrypted credentials.")
+
+
+class LDAPServerConnectionError(UnexpectedError):
+    debug_message_format = _('Unable to establish a connection to '
+                             'LDAP Server (%(url)s).')
+
+
+class LDAPInvalidCredentialsError(UnexpectedError):
+    message_format = _('Unable to authenticate against Identity backend - '
+                       'Invalid username or password')
+
+
+class LDAPSizeLimitExceeded(UnexpectedError):
+    message_format = _('Number of User/Group entities returned by LDAP '
+                       'exceeded size limit. Contact your LDAP '
+                       'administrator.')
+
+
+class CacheDeserializationError(Exception):
+
+    def __init__(self, obj, data):
+        super(CacheDeserializationError, self).__init__(
+            _('Failed to deserialize %(obj)s. Data is %(data)s') % {
+                'obj': obj, 'data': data
+            }
+        )
