@@ -13,32 +13,32 @@
 """Main entry point into the Resource service."""
 
 from oslo_log import log
-from oslo_log import versionutils
 import six
 
 from keystone import assignment
 from keystone.common import cache
 from keystone.common import clean
-from keystone.common import dependency
 from keystone.common import driver_hints
 from keystone.common import manager
+from keystone.common import provider_api
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
-from keystone.i18n import _, _LE, _LW
+from keystone.i18n import _
 from keystone import notifications
 from keystone.resource.backends import base
-from keystone.resource.config_backends import base as config_base
+from keystone.resource.backends import sql as resource_sql
 from keystone.token import provider as token_provider
 
 CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 MEMOIZE = cache.get_memoization_decorator(group='resource')
+PROVIDERS = provider_api.ProviderAPIs
+
+
+TAG_SEARCH_FILTERS = ('tags', 'tags-any', 'not-tags', 'not-tags-any')
 
 
-@dependency.provider('resource_api')
-@dependency.requires('assignment_api', 'credential_api', 'domain_config_api',
-                     'identity_api', 'revoke_api')
 class Manager(manager.Manager):
     """Default pivot point for the Resource backend.
 
@@ -48,27 +48,21 @@ class Manager(manager.Manager):
     """
 
     driver_namespace = 'keystone.resource'
+    _provides_api = 'resource_api'
 
     _DOMAIN = 'domain'
     _PROJECT = 'project'
+    _PROJECT_TAG = 'project tag'
 
     def __init__(self):
-        # If there is a specific driver specified for resource, then use it.
-        # Otherwise retrieve the driver type from the assignment driver.
-        resource_driver = CONF.resource.driver
-
-        if resource_driver is None:
-            assignment_manager = dependency.get_provider('assignment_api')
-            resource_driver = assignment_manager.default_resource_driver()
-
-        super(Manager, self).__init__(resource_driver)
-
-        # Make sure it is a driver version we support, and if it is a legacy
-        # driver, then wrap it.
-        if isinstance(self.driver, base.ResourceDriverV8):
-            self.driver = base.V9ResourceWrapperForV8Driver(self.driver)
-        elif not isinstance(self.driver, base.ResourceDriverV9):
-            raise exception.UnsupportedDriverVersion(driver=resource_driver)
+        # NOTE(morgan): The resource driver must be SQL. This is because there
+        # is a FK between identity and resource. Almost every deployment uses
+        # SQL Identity in some form. Even if SQL Identity is not used, there
+        # is almost no reason to have non-SQL Resource. Keystone requires
+        # SQL in a number of ways, this simply codifies it plainly for resource
+        # the driver_name = None simply implies we don't need to load a driver.
+        self.driver = resource_sql.Resource()
+        super(Manager, self).__init__(driver_name=None)
 
     def _get_hierarchy_depth(self, parents_list):
         return len(parents_list) + 1
@@ -82,6 +76,15 @@ class Manager(manager.Manager):
         # pushing any existing hierarchies over the limit, we add one to the
         # maximum depth allowed, as specified in the configuration file.
         max_depth = CONF.max_project_tree_depth + 1
+
+        # NOTE(wxy): If the hierarchical limit enforcement model is used, the
+        # project depth should be not greater than the model's limit as well.
+        #
+        # TODO(wxy): Deprecate and remove CONF.max_project_tree_depth, let the
+        # depth check only based on the limit enforcement model.
+        limit_model = PROVIDERS.unified_limit_api.enforcement_model
+        if limit_model.MAX_PROJECT_TREE_DEPTH is not None:
+            max_depth = min(max_depth, limit_model.MAX_PROJECT_TREE_DEPTH + 1)
         if self._get_hierarchy_depth(parents_list) > max_depth:
             raise exception.ForbiddenNotSecurity(
                 _('Max hierarchy depth reached for %s branch.') % project_id)
@@ -98,7 +101,7 @@ class Manager(manager.Manager):
         :raises keystone.exception.ValidationError: If one of the constraints
             was not satisfied.
         """
-        if (not self.identity_api.multiple_domains_supported and
+        if (not PROVIDERS.identity_api.multiple_domains_supported and
                 project_ref['id'] != CONF.identity.default_domain_id):
             raise exception.ValidationError(
                 message=_('Multiple domains are not supported'))
@@ -130,19 +133,18 @@ class Manager(manager.Manager):
         if parent_ref['is_domain']:
             if parent_ref['id'] != domain['id']:
                 raise exception.ValidationError(
-                    message=_('Cannot create project, since its parent '
-                              '(%(domain_id)s) is acting as a domain, '
-                              'but project\'s specified parent_id '
-                              '(%(parent_id)s) does not match '
-                              'this domain_id.')
-                    % {'domain_id': domain['id'],
-                       'parent_id': parent_ref['id']})
+                    message=_('Cannot create project, the parent '
+                              '(%(parent_id)s) is acting as a domain, '
+                              'but this project\'s domain id (%(domain_id)s) '
+                              'does not match the parent\'s id.')
+                    % {'parent_id': parent_ref['id'],
+                       'domain_id': domain['id']})
         else:
             parent_domain_id = parent_ref.get('domain_id')
             if parent_domain_id != domain['id']:
                 raise exception.ValidationError(
                     message=_('Cannot create project, since it specifies '
-                              'its owner as domain %(domain_id)s, but '
+                              'its domain_id %(domain_id)s, but '
                               'specifies a parent in a different domain '
                               '(%(parent_domain_id)s).')
                     % {'domain_id': domain['id'],
@@ -230,6 +232,9 @@ class Manager(manager.Manager):
             self.get_project.set(ret, self, project_id)
             self.get_project_by_name.set(ret, self, ret['name'],
                                          ret['domain_id'])
+
+        assignment.COMPUTED_ASSIGNMENTS_REGION.invalidate()
+
         return ret
 
     def assert_domain_enabled(self, domain_id, domain=None):
@@ -296,6 +301,7 @@ class Manager(manager.Manager):
         # Use the driver directly to prevent using old cached value.
         original_project = self.driver.get_project(project_id)
         project = project.copy()
+        self._require_matching_domain_id(project, original_project)
 
         if original_project['is_domain']:
             domain = self._get_domain_from_project(original_project)
@@ -327,38 +333,6 @@ class Manager(manager.Manager):
             raise exception.ValidationError(
                 message=_('Update of `is_domain` is not allowed.'))
 
-        update_domain = ('domain_id' in project and
-                         project['domain_id'] != original_project['domain_id'])
-
-        # NOTE(htruta): Even if we are allowing domain_ids to be
-        # modified (i.e. 'domain_id_immutable' is set False),
-        # a project.domain_id can only be updated for root projects
-        # that have no children. The update of domain_id of a project in
-        # the middle of the hierarchy creates an inconsistent project
-        # hierarchy.
-        if update_domain:
-            if original_project['is_domain']:
-                raise exception.ValidationError(
-                    message=_('Update of domain_id of projects acting as '
-                              'domains is not allowed.'))
-            parent_project = (
-                self.driver.get_project(original_project['parent_id']))
-            is_root_project = parent_project['is_domain']
-            if not is_root_project:
-                raise exception.ValidationError(
-                    message=_('Update of domain_id is only allowed for '
-                              'root projects.'))
-            subtree_list = self.list_projects_in_subtree(project_id)
-            if subtree_list:
-                raise exception.ValidationError(
-                    message=_('Cannot update domain_id of a project that '
-                              'has children.'))
-            versionutils.report_deprecated_feature(
-                LOG,
-                _('update of domain_id is deprecated as of Mitaka '
-                  'and will be removed in O.')
-            )
-
         original_project_enabled = original_project.get('enabled', True)
         project_enabled = project.get('enabled', True)
         if not original_project_enabled and project_enabled:
@@ -379,6 +353,11 @@ class Manager(manager.Manager):
 
             notifications.Audit.disabled(self._PROJECT, project_id,
                                          public=False)
+            # Drop the computed assignments if the project is being disabled.
+            # This ensures an accurate list of projects is returned when
+            # listing projects/domains for a user based on role assignments.
+            assignment.COMPUTED_ASSIGNMENTS_REGION.invalidate()
+
         if cascade:
             self._only_allow_enabled_to_update_cascade(project,
                                                        original_project)
@@ -460,28 +439,20 @@ class Manager(manager.Manager):
 
         return ret
 
-    def _pre_delete_cleanup_project(self, project_id, project, initiator=None):
-        project_user_ids = (
-            self.assignment_api.list_user_ids_for_project(project_id))
-        for user_id in project_user_ids:
-            payload = {'user_id': user_id, 'project_id': project_id}
-            notifications.Audit.internal(
-                notifications.INVALIDATE_USER_PROJECT_TOKEN_PERSISTENCE,
-                payload
-            )
-
     def _post_delete_cleanup_project(self, project_id, project,
                                      initiator=None):
         try:
             self.get_project.invalidate(self, project_id)
             self.get_project_by_name.invalidate(self, project['name'],
                                                 project['domain_id'])
-            self.assignment_api.delete_project_assignments(project_id)
+            PROVIDERS.assignment_api.delete_project_assignments(project_id)
             # Invalidate user role assignments cache region, as it may
             # be caching role assignments where the target is
             # the specified project
             assignment.COMPUTED_ASSIGNMENTS_REGION.invalidate()
-            self.credential_api.delete_credentials_for_project(project_id)
+            PROVIDERS.credential_api.delete_credentials_for_project(project_id)
+            PROVIDERS.trust_api.delete_trusts_for_project(project_id)
+            PROVIDERS.unified_limit_api.delete_limits_for_project(project_id)
         finally:
             # attempt to send audit event even if the cache invalidation raises
             notifications.Audit.deleted(self._PROJECT, project_id, initiator)
@@ -533,20 +504,26 @@ class Manager(manager.Manager):
             project_list = subtree_list + [project]
             projects_ids = [x['id'] for x in project_list]
 
-            for prj in project_list:
-                self._pre_delete_cleanup_project(prj['id'], prj, initiator)
             ret = self.driver.delete_projects_from_ids(projects_ids)
             for prj in project_list:
                 self._post_delete_cleanup_project(prj['id'], prj, initiator)
         else:
-            self._pre_delete_cleanup_project(project_id, project, initiator)
             ret = self.driver.delete_project(project_id)
             self._post_delete_cleanup_project(project_id, project, initiator)
 
+        reason = (
+            'The token cache is being invalidate because project '
+            '%(project_id)s was deleted. Authorization will be recalculated '
+            'and enforced accordingly the next time users authenticate or '
+            'validate a token.' % {'project_id': project_id}
+        )
+        notifications.invalidate_token_cache_notification(reason)
         return ret
 
     def _filter_projects_list(self, projects_list, user_id):
-        user_projects = self.assignment_api.list_projects_for_user(user_id)
+        user_projects = PROVIDERS.assignment_api.list_projects_for_user(
+            user_id
+        )
         user_projects_ids = set([proj['id'] for proj in user_projects])
         # Keep only the projects present in user_projects
         return [proj for proj in projects_list
@@ -559,13 +536,28 @@ class Manager(manager.Manager):
         # Check if project_id exists
         self.get_project(project_id)
 
-    def list_project_parents(self, project_id, user_id=None):
+    def _include_limits(self, projects):
+        """Modify a list of projects to include limit information.
+
+        :param projects: a list of project references including an `id`
+        :type projects: list of dictionaries
+        """
+        for project in projects:
+            hints = driver_hints.Hints()
+            hints.add_filter('project_id', project['id'])
+            limits = PROVIDERS.unified_limit_api.list_limits(hints)
+            project['limits'] = limits
+
+    def list_project_parents(self, project_id, user_id=None,
+                             include_limits=False):
         self._assert_valid_project_id(project_id)
         parents = self.driver.list_project_parents(project_id)
         # If a user_id was provided, the returned list should be filtered
         # against the projects this user has access to.
         if user_id:
             parents = self._filter_projects_list(parents, user_id)
+        if include_limits:
+            self._include_limits(parents)
         return parents
 
     def _build_parents_as_ids_dict(self, project, parents_by_id):
@@ -611,13 +603,16 @@ class Manager(manager.Manager):
             project, {proj['id']: proj for proj in parents_list})
         return parents_as_ids
 
-    def list_projects_in_subtree(self, project_id, user_id=None):
+    def list_projects_in_subtree(self, project_id, user_id=None,
+                                 include_limits=False):
         self._assert_valid_project_id(project_id)
         subtree = self.driver.list_projects_in_subtree(project_id)
         # If a user_id was provided, the returned list should be filtered
         # against the projects this user has access to.
         if user_id:
             subtree = self._filter_projects_list(subtree, user_id)
+        if include_limits:
+            self._include_limits(subtree)
         return subtree
 
     def _build_subtree_as_ids_dict(self, project_id, subtree_by_parent):
@@ -732,9 +727,9 @@ class Manager(manager.Manager):
         result can be returned in response to a domain API call.
         """
         if not project_ref['is_domain']:
-            LOG.error(_LE('Asked to convert a non-domain project into a '
-                          'domain - Domain: %(domain_id)s, Project ID: '
-                          '%(id)s, Project Name: %(project_name)s'),
+            LOG.error('Asked to convert a non-domain project into a '
+                      'domain - Domain: %(domain_id)s, Project ID: '
+                      '%(id)s, Project Name: %(project_name)s',
                       {'domain_id': project_ref['domain_id'],
                        'id': project_ref['id'],
                        'project_name': project_ref['name']})
@@ -803,13 +798,16 @@ class Manager(manager.Manager):
                   'first.'))
 
         self._delete_domain_contents(domain_id)
+        notifications.Audit.internal(
+            notifications.DOMAIN_DELETED, domain_id
+        )
         self._delete_project(domain_id, initiator)
         try:
             self.get_domain.invalidate(self, domain_id)
             self.get_domain_by_name.invalidate(self, domain['name'])
             # Delete any database stored domain config
-            self.domain_config_api.delete_config_options(domain_id)
-            self.domain_config_api.release_registration(domain_id)
+            PROVIDERS.domain_config_api.delete_config_options(domain_id)
+            PROVIDERS.domain_config_api.release_registration(domain_id)
         finally:
             # attempt to send audit event even if the cache invalidation raises
             notifications.Audit.deleted(self._DOMAIN, domain_id, initiator)
@@ -826,8 +824,8 @@ class Manager(manager.Manager):
         """
         def _delete_projects(project, projects, examined):
             if project['id'] in examined:
-                msg = _LE('Circular reference or a repeated entry found '
-                          'projects hierarchy - %(project_id)s.')
+                msg = ('Circular reference or a repeated entry found '
+                       'projects hierarchy - %(project_id)s.')
                 LOG.error(msg, {'project_id': project['id']})
                 return
 
@@ -856,6 +854,18 @@ class Manager(manager.Manager):
 
     @manager.response_truncated
     def list_projects(self, hints=None):
+        if hints:
+            tag_filters = {}
+            # Handle project tag filters separately
+            for f in list(hints.filters):
+                if f['name'] in TAG_SEARCH_FILTERS:
+                    tag_filters[f['name']] = f['value']
+                    hints.filters.remove(f)
+            if tag_filters:
+                tag_refs = self.driver.list_projects_by_tags(tag_filters)
+                project_refs = self.driver.list_projects(hints)
+                ref_ids = [ref['id'] for ref in tag_refs]
+                return [ref for ref in project_refs if ref['id'] in ref_ids]
         return self.driver.list_projects(hints or driver_hints.Hints())
 
     # NOTE(henry-nash): list_projects_in_domain is actually an internal method
@@ -876,76 +886,107 @@ class Manager(manager.Manager):
     def get_project_by_name(self, project_name, domain_id):
         return self.driver.get_project_by_name(project_name, domain_id)
 
-    def ensure_default_domain_exists(self):
-        """Create the default domain if it doesn't exist.
+    def _require_matching_domain_id(self, new_ref, orig_ref):
+        """Ensure the current domain ID matches the reference one, if any.
 
-        This is only used for the v2 API and can go away when V2 does.
+        Provided we want domain IDs to be immutable, check whether any
+        domain_id specified in the ref dictionary matches the existing
+        domain_id for this entity.
 
+        :param new_ref: the dictionary of new values proposed for this entity
+        :param orig_ref: the dictionary of original values proposed for this
+                         entity
+        :raises: :class:`keystone.exception.ValidationError`
         """
+        if 'domain_id' in new_ref:
+            if new_ref['domain_id'] != orig_ref['domain_id']:
+                raise exception.ValidationError(_('Cannot change Domain ID'))
+
+    def create_project_tag(self, project_id, tag, initiator=None):
+        """Create a new tag on project.
+
+        :param project_id: ID of a project to create a tag for
+        :param tag: The string value of a tag to add
+
+        :returns: The value of the created tag
+        """
+        project = self.driver.get_project(project_id)
+        tag_name = tag.strip()
+        project['tags'].append(tag_name)
+        self.update_project(project_id, {'tags': project['tags']})
+
+        notifications.Audit.created(
+            self._PROJECT_TAG, tag_name, initiator)
+        return tag_name
+
+    def get_project_tag(self, project_id, tag_name):
+        """Return information for a single tag on a project.
+
+        :param project_id: ID of a project to retrive a tag from
+        :param tag_name: Name of a tag to return
+
+        :raises keystone.exception.ProjectTagNotFound: If the tag name
+            does not exist on the project
+        :returns: The tag value
+        """
+        project = self.driver.get_project(project_id)
+        if tag_name not in project.get('tags'):
+            raise exception.ProjectTagNotFound(project_tag=tag_name)
+        return tag_name
+
+    def list_project_tags(self, project_id):
+        """List all tags on project.
+
+        :param project_id: The ID of a project
+
+        :returns: A list of tags from a project
+        """
+        project = self.driver.get_project(project_id)
+        return project.get('tags', [])
+
+    def update_project_tags(self, project_id, tags, initiator=None):
+        """Update all tags on a project.
+
+        :param project_id: The ID of the project to update
+        :param tags: A list of tags to update on the project
+
+        :returns: A list of tags
+        """
+        self.driver.get_project(project_id)
+        tag_list = [t.strip() for t in tags]
+        project = {'tags': tag_list}
+        self.update_project(project_id, project)
+        return tag_list
+
+    def delete_project_tag(self, project_id, tag):
+        """Delete single tag from project.
+
+        :param project_id: The ID of the project
+        :param tag: The tag value to delete
+
+        :raises keystone.exception.ProjectTagNotFound: If the tag name
+            does not exist on the project
+        """
+        project = self.driver.get_project(project_id)
         try:
-            default_domain_attrs = {
-                'name': 'Default',
-                'id': CONF.identity.default_domain_id,
-                'description': 'Domain created automatically to support V2.0 '
-                               'operations.',
-            }
-            self.create_domain(CONF.identity.default_domain_id,
-                               default_domain_attrs)
-            LOG.warning(_LW(
-                'The default domain was created automatically to contain V2 '
-                'resources. This is deprecated in the M release and will not '
-                'be supported in the O release. Create the default domain '
-                'manually or use the keystone-manage bootstrap command.'))
-        except exception.Conflict:
-            LOG.debug('The default domain already exists.')
-        except Exception:
-            LOG.error(_LE('Failed to create the default domain.'))
-            raise
+            project['tags'].remove(tag)
+        except ValueError:
+            raise exception.ProjectTagNotFound(project_tag=tag)
+        self.update_project(project_id, project)
+        notifications.Audit.deleted(self._PROJECT_TAG, tag)
 
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.resource.ResourceDriverBase',
-    in_favor_of='keystone.resource.backends.base.ResourceDriverBase',
-    remove_in=+1)
-class ResourceDriverBase(base.ResourceDriverBase):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.resource.ResourceDriverV8',
-    in_favor_of='keystone.resource.backends.base.ResourceDriverV8',
-    remove_in=+1)
-class ResourceDriverV8(base.ResourceDriverV8):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.resource.ResourceDriverV9',
-    in_favor_of='keystone.resource.backends.base.ResourceDriverV9',
-    remove_in=+1)
-class ResourceDriverV9(base.ResourceDriverV9):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.resource.V9ResourceWrapperForV8Driver',
-    in_favor_of='keystone.resource.backends.base.V9ResourceWrapperForV8Driver',
-    remove_in=+1)
-class V9ResourceWrapperForV8Driver(base.V9ResourceWrapperForV8Driver):
-    pass
-
-
-Driver = manager.create_legacy_driver(base.ResourceDriverV8)
+    def check_project_depth(self, max_depth=None):
+        """Check project depth whether greater than input or not."""
+        if max_depth:
+            exceeded_project_ids = self.driver.check_project_depth(max_depth)
+            if exceeded_project_ids:
+                raise exception.LimitTreeExceedError(exceeded_project_ids,
+                                                     max_depth)
 
 
 MEMOIZE_CONFIG = cache.get_memoization_decorator(group='domain_config')
 
 
-@dependency.provider('domain_config_api')
 class DomainConfigManager(manager.Manager):
     """Default pivot point for the Domain Config backend."""
 
@@ -959,12 +1000,17 @@ class DomainConfigManager(manager.Manager):
     # the identity manager are supported.
 
     driver_namespace = 'keystone.resource.domain_config'
+    _provides_api = 'domain_config_api'
 
+    # We explicitly state each whitelisted option instead of pulling all ldap
+    # options from CONF and selectively pruning them to prevent a security
+    # lapse. That way if a new ldap CONF key/value were to be added it wouldn't
+    # automatically be added to the whitelisted options unless that is what was
+    # intended. In which case, we explicitly add it to the list ourselves.
     whitelisted_options = {
         'identity': ['driver', 'list_limit'],
         'ldap': [
-            'url', 'user', 'suffix', 'use_dumb_member', 'dumb_member',
-            'allow_subtree_delete', 'query_scope', 'page_size',
+            'url', 'user', 'suffix', 'query_scope', 'page_size',
             'alias_dereferencing', 'debug_level', 'chase_referrals',
             'user_tree_dn', 'user_filter', 'user_objectclass',
             'user_id_attribute', 'user_name_attribute', 'user_mail_attribute',
@@ -972,17 +1018,15 @@ class DomainConfigManager(manager.Manager):
             'user_enabled_attribute', 'user_enabled_invert',
             'user_enabled_mask', 'user_enabled_default',
             'user_attribute_ignore', 'user_default_project_id_attribute',
-            'user_allow_create', 'user_allow_update', 'user_allow_delete',
             'user_enabled_emulation', 'user_enabled_emulation_dn',
             'user_enabled_emulation_use_group_config',
             'user_additional_attribute_mapping', 'group_tree_dn',
             'group_filter', 'group_objectclass', 'group_id_attribute',
-            'group_name_attribute', 'group_member_attribute',
-            'group_desc_attribute', 'group_attribute_ignore',
-            'group_allow_create', 'group_allow_update', 'group_allow_delete',
-            'group_additional_attribute_mapping', 'tls_cacertfile',
-            'tls_cacertdir', 'use_tls', 'tls_req_cert', 'use_pool',
-            'pool_size', 'pool_retry_max', 'pool_retry_delay',
+            'group_name_attribute', 'group_members_are_ids',
+            'group_member_attribute', 'group_desc_attribute',
+            'group_attribute_ignore', 'group_additional_attribute_mapping',
+            'tls_cacertfile', 'tls_cacertdir', 'use_tls', 'tls_req_cert',
+            'use_pool', 'pool_size', 'pool_retry_max', 'pool_retry_delay',
             'pool_connection_timeout', 'pool_connection_lifetime',
             'use_auth_pool', 'auth_pool_size', 'auth_pool_connection_lifetime'
         ]
@@ -1071,6 +1115,11 @@ class DomainConfigManager(manager.Manager):
 
         return option_list
 
+    def _option_dict(self, group, option):
+        group_attr = getattr(CONF, group)
+        return {'group': group, 'option': option,
+                'value': getattr(group_attr, option)}
+
     def _list_to_config(self, whitelisted, sensitive=None, req_option=None):
         """Build config dict from a list of option dicts.
 
@@ -1095,9 +1144,9 @@ class DomainConfigManager(manager.Manager):
             # there is only one option in the answer (and that it's the right
             # one) - if not, something has gone wrong and we raise an error
             if len(the_list) > 1 or the_list[0]['option'] != req_option:
-                LOG.error(_LE('Unexpected results in response for domain '
-                              'config - %(count)s responses, first option is '
-                              '%(option)s, expected option %(expected)s'),
+                LOG.error('Unexpected results in response for domain '
+                          'config - %(count)s responses, first option is '
+                          '%(option)s, expected option %(expected)s',
                           {'count': len(the_list), 'option': list[0]['option'],
                            'expected': req_option})
                 raise exception.UnexpectedError(
@@ -1179,6 +1228,53 @@ class DomainConfigManager(manager.Manager):
         raise exception.DomainConfigNotFound(
             domain_id=domain_id, group_or_option=msg)
 
+    def get_security_compliance_config(self, domain_id, group, option=None):
+        r"""Get full or partial security compliance config from configuration.
+
+        :param domain_id: the domain in question
+        :param group: a specific group of options
+        :param option: an optional specific option within the group
+
+        :returns: a dict of group dicts containing the whitelisted options,
+                  filtered by group and option specified
+        :raises keystone.exception.InvalidDomainConfig: when the config
+                and group/option parameters specify an option we do not
+                support
+
+        An example response::
+
+            {
+                'security_compliance': {
+                    'password_regex': '^(?=.*\d)(?=.*[a-zA-Z]).{7,}$'
+                    'password_regex_description':
+                        'A password must consist of at least 1 letter, '
+                        '1 digit, and have a minimum length of 7 characters'
+                    }
+            }
+
+        """
+        if domain_id != CONF.identity.default_domain_id:
+            msg = _('Reading security compliance information for any domain '
+                    'other than the default domain is not allowed or '
+                    'supported.')
+            raise exception.InvalidDomainConfig(reason=msg)
+
+        config_list = []
+        readable_options = ['password_regex', 'password_regex_description']
+        if option and option not in readable_options:
+            msg = _('Reading security compliance values other than '
+                    'password_regex and password_regex_description is not '
+                    'allowed.')
+            raise exception.InvalidDomainConfig(reason=msg)
+        elif option and option in readable_options:
+            config_list.append(self._option_dict(group, option))
+        elif not option:
+            for op in readable_options:
+                config_list.append(self._option_dict(group, op))
+        # We already validated that the group is the security_compliance group
+        # so we can move along and start validating the options
+        return self._list_to_config(config_list, req_option=option)
+
     def update_config(self, domain_id, config, group=None, option=None):
         """Update config, or partial config, for a domain.
 
@@ -1354,21 +1450,21 @@ class DomainConfigManager(manager.Manager):
                 each_whitelisted['value'] = (
                     each_whitelisted['value'] % sensitive_dict)
             except KeyError:
-                warning_msg = _LW(
+                warning_msg = (
                     'Found what looks like an unmatched config option '
                     'substitution reference - domain: %(domain)s, group: '
                     '%(group)s, option: %(option)s, value: %(value)s. Perhaps '
                     'the config option to which it refers has yet to be '
                     'added?')
             except (ValueError, TypeError):
-                warning_msg = _LW(
+                warning_msg = (
                     'Found what looks like an incorrectly constructed '
                     'config option substitution reference - domain: '
                     '%(domain)s, group: %(group)s, option: %(option)s, '
                     'value: %(value)s.')
 
             if warning_msg:
-                LOG.warning(warning_msg % {
+                LOG.warning(warning_msg, {
                     'domain': domain_id,
                     'group': each_whitelisted['group'],
                     'option': each_whitelisted['option'],
@@ -1412,14 +1508,6 @@ class DomainConfigManager(manager.Manager):
             }
 
         """
-        def _option_dict(group, option):
-            group_attr = getattr(CONF, group)
-            if group_attr is None:
-                msg = _('Group  %s not found in config') % group
-                raise exception.UnexpectedError(msg)
-            return {'group': group, 'option': option,
-                    'value': getattr(group_attr, option)}
-
         self._assert_valid_group_and_option(group, option)
         config_list = []
         if group:
@@ -1429,26 +1517,15 @@ class DomainConfigManager(manager.Manager):
                             'group %(group)s is not supported') % {
                                 'option': option, 'group': group}
                     raise exception.InvalidDomainConfig(reason=msg)
-                config_list.append(_option_dict(group, option))
+                config_list.append(self._option_dict(group, option))
             else:
                 for each_option in self.whitelisted_options[group]:
-                    config_list.append(_option_dict(group, each_option))
+                    config_list.append(self._option_dict(group, each_option))
         else:
             for each_group in self.whitelisted_options:
                 for each_option in self.whitelisted_options[each_group]:
-                    config_list.append(_option_dict(each_group, each_option))
+                    config_list.append(
+                        self._option_dict(each_group, each_option)
+                    )
 
         return self._list_to_config(config_list, req_option=option)
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.resource.DomainConfigDriverV8',
-    in_favor_of='keystone.resource.config_backends.base.DomainConfigDriverV8',
-    remove_in=+1)
-class DomainConfigDriverV8(config_base.DomainConfigDriverV8):
-    pass
-
-
-DomainConfigDriver = manager.create_legacy_driver(
-    config_base.DomainConfigDriverV8)
