@@ -18,7 +18,6 @@ import uuid
 import freezegun
 from oslo_config import fixture as config_fixture
 from oslo_log import log
-from oslo_serialization import jsonutils
 import six
 
 from keystone.common import fernet_utils
@@ -26,10 +25,10 @@ from keystone.common import utils as common_utils
 import keystone.conf
 from keystone.credential.providers import fernet as credential_fernet
 from keystone import exception
+from keystone.server.flask import application
 from keystone.tests import unit
 from keystone.tests.unit import ksfixtures
 from keystone.tests.unit import utils
-from keystone.version import service
 
 
 CONF = keystone.conf.CONF
@@ -72,6 +71,34 @@ class UtilsTestCase(unit.BaseTestCase):
         self.assertRaises(ValueError, common_utils.resource_uuid,
                           value)
 
+    def test_remove_duplicate_dicts_from_list(self):
+        dict_list = []
+        num_of_duplicate = 10
+        dup_value = {
+            'id': uuid.uuid4().hex,
+            'name': uuid.uuid4().hex,
+            'tags': ['foo', 'bar']
+        }
+        # Add in 10 unique items, and 10 of the same item
+        for i in range(num_of_duplicate):
+            new_dict = {
+                'id': i,
+                'name': uuid.uuid4().hex,
+                'tags': ['foo', i]
+            }
+            dict_list.append(new_dict)
+            dict_list.append(dup_value)
+        self.assertEqual(len(dict_list), 20)
+        result = common_utils.remove_duplicate_dicts_by_id(dict_list)
+        # Assert that 9 duplicate items have been removed
+        self.assertEqual(len(result), len(dict_list) - 9)
+        # Show that the duplicate item is only stored once
+        count = 0
+        for r in result:
+            if r['id'] == dup_value['id']:
+                count += 1
+        self.assertEqual(count, 1)
+
     def test_hash(self):
         password = 'right'
         wrong = 'wrongwrong'  # Two wrongs don't make a right
@@ -113,6 +140,19 @@ class UtilsTestCase(unit.BaseTestCase):
                           common_utils.verify_length_and_trunc_password,
                           invalid_password)
 
+    def test_verify_length_and_trunc_password_throws_validation_error(self):
+        class SpecialObject(object):
+            pass
+
+        special_object = SpecialObject()
+        invalid_passwords = [True, special_object, 4.3, 5]
+        for invalid_password in invalid_passwords:
+            self.assertRaises(
+                exception.ValidationError,
+                common_utils.verify_length_and_trunc_password,
+                invalid_password
+            )
+
     def test_hash_long_password_truncation(self):
         self.config_fixture.config(strict_password_check=False)
         invalid_length_password = '0' * 9999999
@@ -169,26 +209,6 @@ class UtilsTestCase(unit.BaseTestCase):
         self.assertFalse(common_utils.auth_str_equal('aaaaa', 'a'))
         self.assertFalse(common_utils.auth_str_equal('ABC123', 'abc123'))
 
-    def test_unixtime(self):
-        global TZ
-
-        @utils.timezone
-        def _test_unixtime():
-            epoch = common_utils.unixtime(dt)
-            self.assertEqual(epoch, epoch_ans, "TZ=%s" % TZ)
-
-        dt = datetime.datetime(1970, 1, 2, 3, 4, 56, 0)
-        epoch_ans = 56 + 4 * 60 + 3 * 3600 + 86400
-        for d in ['+0', '-11', '-8', '-5', '+5', '+8', '+14']:
-            TZ = 'UTC' + d
-            _test_unixtime()
-
-    def test_pki_encoder(self):
-        data = {'field': 'value'}
-        json = jsonutils.dumps(data, cls=common_utils.PKIEncoder)
-        expected_json = '{"field":"value"}'
-        self.assertEqual(expected_json, json)
-
     def test_url_safe_check(self):
         base_str = 'i am safe'
         self.assertFalse(common_utils.is_not_url_safe(base_str))
@@ -230,7 +250,7 @@ class UtilsTestCase(unit.BaseTestCase):
 
 class ServiceHelperTests(unit.BaseTestCase):
 
-    @service.fail_gracefully
+    @application.fail_gracefully
     def _do_test(self):
         raise Exception("Test Exc")
 
@@ -255,7 +275,8 @@ class FernetUtilsTestCase(unit.BaseTestCase):
         logging_fixture = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
         fernet_utilities = fernet_utils.FernetUtils(
             CONF.fernet_tokens.key_repository,
-            CONF.fernet_tokens.max_active_keys
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
         )
         fernet_utilities.load_keys()
         expected_debug_message = (
@@ -277,11 +298,12 @@ class FernetUtilsTestCase(unit.BaseTestCase):
         logging_fixture = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
         fernet_utilities = fernet_utils.FernetUtils(
             CONF.credential.key_repository,
-            credential_fernet.MAX_ACTIVE_KEYS
+            credential_fernet.MAX_ACTIVE_KEYS,
+            'credential'
         )
         fernet_utilities.load_keys()
         debug_message = (
-            'Loaded 2 Fernet keys from %(dir)s, but `[fernet_tokens] '
+            'Loaded 2 Fernet keys from %(dir)s, but `[credential] '
             'max_active_keys = %(max)d`; perhaps there have not been enough '
             'key rotations to reach `max_active_keys` yet?') % {
                 'dir': CONF.credential.key_repository,
