@@ -41,9 +41,6 @@ SCOPE_NAMES = {
     ldap.SCOPE_SUBTREE: 'SCOPE_SUBTREE',
 }
 
-# http://msdn.microsoft.com/en-us/library/windows/desktop/aa366991(v=vs.85).aspx  # noqa
-CONTROL_TREEDELETE = '1.2.840.113556.1.4.805'
-
 LOG = log.getLogger(__name__)
 CONF = keystone.conf.CONF
 
@@ -68,7 +65,7 @@ def _internal_attr(attr_name, value_or_values):
             return 'CN=Doe\\2C John,OU=Users,CN=example,CN=com'
 
         try:
-            dn = ldap.dn.str2dn(common.utf8_encode(dn))
+            dn = ldap.dn.str2dn(dn)
         except ldap.DECODING_ERROR:
             # NOTE(amakarov): In case of IDs instead of DNs in group members
             # they must be handled as regular values.
@@ -77,10 +74,9 @@ def _internal_attr(attr_name, value_or_values):
         norm = []
         for part in dn:
             name, val, i = part[0]
-            name = common.utf8_decode(name)
             name = name.upper()
             norm.append([(name, val, i)])
-        return common.utf8_decode(ldap.dn.dn2str(norm))
+        return ldap.dn.dn2str(norm)
 
     if attr_name in ('member', 'roleOccupant'):
         attr_fn = normalize_dn
@@ -169,9 +165,9 @@ def _match(key, value, attrs):
     if value == '*':
         return True
     if key == 'serviceId':
-        # for serviceId, the backend is returning a list of numbers
-        # make sure we convert them to strings first before comparing
-        # them
+        # For serviceId, the backend is returning a list of numbers.
+        # Make sure we convert them to strings first before comparing
+        # them.
         str_sids = [six.text_type(x) for x in attrs[key]]
         return six.text_type(value) in str_sids
     if key != 'objectclass':
@@ -179,7 +175,7 @@ def _match(key, value, attrs):
         norm_values = list(
             _internal_attr(key, x)[0].lower() for x in attrs[key])
         return match_with_wildcards(check_value, norm_values)
-    # it is an objectclass check, so check subclasses
+    # It is an objectclass check, so check subclasses
     values = _subs(value)
     for v in values:
         if v in attrs[key]:
@@ -219,8 +215,8 @@ PendingRequests = {}
 class FakeLdap(common.LDAPHandler):
     """Emulate the python-ldap API.
 
-    The python-ldap API requires all strings to be UTF-8 encoded. This
-    is assured by the caller of this interface
+    The python-ldap API requires all strings to be UTF-8 encoded with the
+    exception of [1]. This is assured by the caller of this interface
     (i.e. KeystoneLDAPHandler).
 
     However, internally this emulation MUST process and store strings
@@ -231,6 +227,11 @@ class FakeLdap(common.LDAPHandler):
     emulation, and encodes them back to UTF-8 when returning values
     from the emulation.
 
+    [1] Some fields (DNs, RDNs, attribute names, queries) are represented
+    as text in python-ldap for Python 3, and for Python 2 when
+    bytes_mode=False. For more details see:
+    http://www.python-ldap.org/en/latest/bytes_mode.html#bytes-mode
+
     """
 
     __prefix = 'ldap:'
@@ -244,7 +245,8 @@ class FakeLdap(common.LDAPHandler):
                 tls_req_cert='demand', chase_referrals=None, debug_level=None,
                 use_pool=None, pool_size=None, pool_retry_max=None,
                 pool_retry_delay=None, pool_conn_timeout=None,
-                pool_conn_lifetime=None):
+                pool_conn_lifetime=None,
+                conn_timeout=None):
         if url.startswith('fake://memory'):
             if url not in FakeShelves:
                 FakeShelves[url] = FakeShelve()
@@ -278,20 +280,16 @@ class FakeLdap(common.LDAPHandler):
         self.pool_retry_delay = pool_retry_delay
         self.pool_conn_timeout = pool_conn_timeout
         self.pool_conn_lifetime = pool_conn_lifetime
-
-    def dn(self, dn):
-        return common.utf8_decode(dn)
+        self.conn_timeout = conn_timeout
 
     def _dn_to_id_attr(self, dn):
-        return common.utf8_decode(
-            ldap.dn.str2dn(common.utf8_encode(dn))[0][0][0])
+        return ldap.dn.str2dn(dn)[0][0][0]
 
     def _dn_to_id_value(self, dn):
-        return common.utf8_decode(
-            ldap.dn.str2dn(common.utf8_encode(dn))[0][0][1])
+        return ldap.dn.str2dn(dn)[0][0][1]
 
     def key(self, dn):
-        return '%s%s' % (self.__prefix, self.dn(dn))
+        return '%s%s' % (self.__prefix, dn)
 
     def simple_bind_s(self, who='', cred='',
                       serverctrls=None, clientctrls=None):
@@ -299,27 +297,23 @@ class FakeLdap(common.LDAPHandler):
         if server_fail:
             raise ldap.SERVER_DOWN
         whos = ['cn=Admin', CONF.ldap.user]
-        if (common.utf8_decode(who) in whos and
-                common.utf8_decode(cred) in ['password', CONF.ldap.password]):
+        if (who in whos and cred in ['password', CONF.ldap.password]):
             return
 
-        try:
-            attrs = self.db[self.key(who)]
-        except KeyError:
-            LOG.debug('bind fail: who=%s not found', common.utf8_decode(who))
-            raise ldap.NO_SUCH_OBJECT
+        attrs = self.db.get(self.key(who))
+        if not attrs:
+            LOG.debug('who=%s not found, binding anonymously', who)
 
-        db_password = None
-        try:
-            db_password = attrs['userPassword'][0]
-        except (KeyError, IndexError):
-            LOG.debug('bind fail: password for who=%s not found',
-                      common.utf8_decode(who))
-            raise ldap.INAPPROPRIATE_AUTH
+        db_password = ''
+        if attrs:
+            try:
+                db_password = attrs['userPassword'][0]
+            except (KeyError, IndexError):
+                LOG.debug('bind fail: password for who=%s not found', who)
+                raise ldap.INAPPROPRIATE_AUTH
 
-        if cred != common.utf8_encode(db_password):
-            LOG.debug('bind fail: password for who=%s does not match',
-                      common.utf8_decode(who))
+        if cred != db_password:
+            LOG.debug('bind fail: password for who=%s does not match', who)
             raise ldap.INVALID_CREDENTIALS
 
     def unbind_s(self):
@@ -348,15 +342,14 @@ class FakeLdap(common.LDAPHandler):
                         id_attr_in_modlist = True
 
         if not id_attr_in_modlist:
-            LOG.debug('id_attribute=%(attr)s missing, attributes=%(attrs)s' %
+            LOG.debug('id_attribute=%(attr)s missing, attributes=%(attrs)s',
                       {'attr': id_attr, 'attrs': modlist})
             raise ldap.NAMING_VIOLATION
         key = self.key(dn)
         LOG.debug('add item: dn=%(dn)s, attrs=%(attrs)s', {
-            'dn': common.utf8_decode(dn), 'attrs': modlist})
+            'dn': dn, 'attrs': modlist})
         if key in self.db:
-            LOG.debug('add item failed: dn=%s is already in store.',
-                      common.utf8_decode(dn))
+            LOG.debug('add item failed: dn=%s is already in store.', dn)
             raise ldap.ALREADY_EXISTS(dn)
 
         self.db[key] = {k: _internal_attr(k, v) for k, v in modlist}
@@ -370,7 +363,7 @@ class FakeLdap(common.LDAPHandler):
         return [k for k, v in self.db.items()
                 if re.match('%s.*,%s' % (
                             re.escape(self.__prefix),
-                            re.escape(self.dn(dn))), k)]
+                            re.escape(dn)), k)]
 
     def delete_ext_s(self, dn, serverctrls, clientctrls=None):
         """Remove the ldap object at specified dn."""
@@ -378,19 +371,11 @@ class FakeLdap(common.LDAPHandler):
             raise ldap.SERVER_DOWN
 
         try:
-            if CONTROL_TREEDELETE in [c.controlType for c in serverctrls]:
-                LOG.debug('FakeLdap subtree_delete item: dn=%s',
-                          common.utf8_decode(dn))
-                children = self._getChildren(dn)
-                for c in children:
-                    del self.db[c]
-
             key = self.key(dn)
-            LOG.debug('FakeLdap delete item: dn=%s', common.utf8_decode(dn))
+            LOG.debug('FakeLdap delete item: dn=%s', dn)
             del self.db[key]
         except KeyError:
-            LOG.debug('delete item failed: dn=%s not found.',
-                      common.utf8_decode(dn))
+            LOG.debug('delete item failed: dn=%s not found.', dn)
             raise ldap.NO_SUCH_OBJECT
         self.db.sync()
 
@@ -406,12 +391,11 @@ class FakeLdap(common.LDAPHandler):
 
         key = self.key(dn)
         LOG.debug('modify item: dn=%(dn)s attrs=%(attrs)s', {
-            'dn': common.utf8_decode(dn), 'attrs': modlist})
+            'dn': dn, 'attrs': modlist})
         try:
             entry = self.db[key]
         except KeyError:
-            LOG.debug('modify item failed: dn=%s not found.',
-                      common.utf8_decode(dn))
+            LOG.debug('modify item failed: dn=%s not found.', dn)
             raise ldap.NO_SUCH_OBJECT
 
         for cmd, k, v in modlist:
@@ -475,9 +459,9 @@ class FakeLdap(common.LDAPHandler):
             results = [(base, item_dict)]
         elif scope == ldap.SCOPE_SUBTREE:
             # FIXME - LDAP search with SUBTREE scope must return the base
-            # entry, but the code below does _not_.  Unfortunately, there are
+            # entry, but the code below does _not_. Unfortunately, there are
             # several tests that depend on this broken behavior, and fail
-            # when the base entry is returned in the search results.  The
+            # when the base entry is returned in the search results. The
             # fix is easy here, just initialize results as above for
             # the SCOPE_BASE case.
             # https://bugs.launchpad.net/keystone/+bug/1368772
@@ -491,19 +475,19 @@ class FakeLdap(common.LDAPHandler):
                             for k, v in self.db.items()
                             if re.match('%s.*,%s' %
                                         (re.escape(self.__prefix),
-                                         re.escape(self.dn(base))), k)]
+                                         re.escape(base)), k)]
             results.extend(extraresults)
         elif scope == ldap.SCOPE_ONELEVEL:
 
             def get_entries():
-                base_dn = ldap.dn.str2dn(common.utf8_encode(base))
+                base_dn = ldap.dn.str2dn(base)
                 base_len = len(base_dn)
 
                 for k, v in self.db.items():
                     if not k.startswith(self.__prefix):
                         continue
                     k_dn_str = k[len(self.__prefix):]
-                    k_dn = ldap.dn.str2dn(common.utf8_encode(k_dn_str))
+                    k_dn = ldap.dn.str2dn(k_dn_str)
                     if len(k_dn) != base_len + 1:
                         continue
                     if k_dn[-base_len:] != base_dn:
@@ -519,13 +503,11 @@ class FakeLdap(common.LDAPHandler):
         objects = []
         for dn, attrs in results:
             # filter the objects by filterstr
-            id_attr, id_val, _ = ldap.dn.str2dn(common.utf8_encode(dn))[0][0]
-            id_attr = common.utf8_decode(id_attr)
-            id_val = common.utf8_decode(id_val)
+            id_attr, id_val, _ = ldap.dn.str2dn(dn)[0][0]
             match_attrs = attrs.copy()
             match_attrs[id_attr] = [id_val]
             attrs_checked = set()
-            if not filterstr or _match_query(common.utf8_decode(filterstr),
+            if not filterstr or _match_query(filterstr,
                                              match_attrs,
                                              attrs_checked):
                 if (filterstr and
@@ -534,7 +516,7 @@ class FakeLdap(common.LDAPHandler):
                     raise AssertionError('No objectClass in search filter')
                 # filter the attributes by attrlist
                 attrs = {k: v for k, v in attrs.items()
-                         if not attrlist or k in common.utf8_decode(attrlist)}
+                         if not attrlist or k in attrlist}
                 objects.append((dn, attrs))
 
         return objects
@@ -658,8 +640,7 @@ class FakeLdapNoSubtreeDelete(FakeLdap):
                 raise ldap.NOT_ALLOWED_ON_NONLEAF
 
         except KeyError:
-            LOG.debug('delete item failed: dn=%s not found.',
-                      common.utf8_decode(dn))
+            LOG.debug('delete item failed: dn=%s not found.', dn)
             raise ldap.NO_SUCH_OBJECT
         super(FakeLdapNoSubtreeDelete, self).delete_ext_s(dn,
                                                           serverctrls,
