@@ -14,7 +14,7 @@
 import fixtures
 from oslo_policy import opts
 
-from keystone.policy.backends import rules
+from keystone.common.rbac_enforcer import policy
 
 
 class Policy(fixtures.Fixture):
@@ -29,5 +29,4 @@ class Policy(fixtures.Fixture):
         opts.set_defaults(self._config_fixture.conf)
         self._config_fixture.config(group='oslo_policy',
                                     policy_file=self._policy_file)
-        rules.init()
-        self.addCleanup(rules.reset)
+        self.addCleanup(policy.reset)
