@@ -281,6 +281,31 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
             user_id,
             bootstrap.password)
 
+    def test_bootstrap_with_ambiguous_role_names(self):
+        bootstrap = cli.BootStrap()
+        # bootstrap system to create the default admin role
+        self._do_test_bootstrap(bootstrap)
+
+        # create a domain-specific roles that share the same names as the
+        # default roles created by keystone-manage bootstrap
+        domain = {'id': uuid.uuid4().hex, 'name': uuid.uuid4().hex}
+        domain = PROVIDERS.resource_api.create_domain(domain['id'], domain)
+        domain_roles = {}
+
+        for name in ['admin', 'member', 'reader']:
+            domain_role = {
+                'domain_id': domain['id'],
+                'id': uuid.uuid4().hex,
+                'name': name
+            }
+            domain_roles[name] = PROVIDERS.role_api.create_role(
+                domain_role['id'], domain_role
+            )
+
+            # ensure subsequent bootstrap attempts don't fail because of
+            # ambiguity
+            self._do_test_bootstrap(bootstrap)
+
 
 class CliBootStrapTestCaseWithEnvironment(CliBootStrapTestCase):
 
@@ -1627,12 +1652,12 @@ class TestTokenFlush(unit.TestCase):
 
     def test_token_flush_emits_warning(self):
         expected_msg = (
-            'This command is deprecated and no longer needed with the '
-            'development of non-persistent token formats. It will be removed '
-            'in Stein. It is recommended that you remove usage of this '
-            'command or integrate it\'s functionality into a separate tool if '
-            'you are using an out-of-tree provider that relies on persistent '
-            'token storage.'
+            'The token_flush command is deprecated and no longer needed with '
+            'the development of non-persistent token formats. It will be '
+            'removed in Stein. It is recommended that you remove usage of '
+            'this command or integrate it\'s functionality into a separate '
+            'tool if you are using an out-of-tree provider that relies on '
+            'persistent token storage.'
         )
         logging = self.useFixture(fixtures.FakeLogger())
         tf = cli.TokenFlush()
