@@ -12,12 +12,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import datetime
 import uuid
 
-import mock
 import oslo_context.context
 from oslo_serialization import jsonutils
-from oslo_utils import timeutils
 from six.moves import http_client
 from testtools import matchers
 import webtest
@@ -25,109 +24,42 @@ import webtest
 from keystone import auth
 from keystone.common import authorization
 from keystone.common import cache
+from keystone.common import provider_api
 from keystone.common.validation import validators
-import keystone.conf
 from keystone import exception
 from keystone import middleware
-from keystone.middleware import auth as middleware_auth
+from keystone.resource.backends import base as resource_base
 from keystone.tests.common import auth as common_auth
 from keystone.tests import unit
 from keystone.tests.unit import rest
 
 
-CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 DEFAULT_DOMAIN_ID = 'default'
 
 TIME_FORMAT = unit.TIME_FORMAT
 
 
-class AuthTestMixin(object):
-    """To hold auth building helper functions."""
-
-    def build_auth_scope(self, project_id=None, project_name=None,
-                         project_domain_id=None, project_domain_name=None,
-                         domain_id=None, domain_name=None, trust_id=None,
-                         unscoped=None, is_domain=None):
-        scope_data = {}
-        if unscoped:
-            scope_data['unscoped'] = {}
-        if project_id or project_name:
-            scope_data['project'] = {}
-            if project_id:
-                scope_data['project']['id'] = project_id
-            else:
-                scope_data['project']['name'] = project_name
-                if is_domain is not None:
-                    scope_data['is_domain'] = is_domain
-                if project_domain_id or project_domain_name:
-                    project_domain_json = {}
-                    if project_domain_id:
-                        project_domain_json['id'] = project_domain_id
-                    else:
-                        project_domain_json['name'] = project_domain_name
-                    scope_data['project']['domain'] = project_domain_json
-        if domain_id or domain_name:
-            scope_data['domain'] = {}
-            if domain_id:
-                scope_data['domain']['id'] = domain_id
-            else:
-                scope_data['domain']['name'] = domain_name
-        if trust_id:
-            scope_data['OS-TRUST:trust'] = {}
-            scope_data['OS-TRUST:trust']['id'] = trust_id
-        return scope_data
-
-    def build_password_auth(self, user_id=None, username=None,
-                            user_domain_id=None, user_domain_name=None,
-                            password=None):
-        password_data = {'user': {}}
-        if user_id:
-            password_data['user']['id'] = user_id
-        else:
-            password_data['user']['name'] = username
-            if user_domain_id or user_domain_name:
-                password_data['user']['domain'] = {}
-                if user_domain_id:
-                    password_data['user']['domain']['id'] = user_domain_id
-                else:
-                    password_data['user']['domain']['name'] = user_domain_name
-        password_data['user']['password'] = password
-        return password_data
-
-    def build_token_auth(self, token):
-        return {'id': token}
-
-    def build_authentication_request(self, token=None, user_id=None,
-                                     username=None, user_domain_id=None,
-                                     user_domain_name=None, password=None,
-                                     kerberos=False, **kwargs):
-        """Build auth dictionary.
-
-        It will create an auth dictionary based on all the arguments
-        that it receives.
-        """
-        auth_data = {}
-        auth_data['identity'] = {'methods': []}
-        if kerberos:
-            auth_data['identity']['methods'].append('kerberos')
-            auth_data['identity']['kerberos'] = {}
-        if token:
-            auth_data['identity']['methods'].append('token')
-            auth_data['identity']['token'] = self.build_token_auth(token)
-        if user_id or username:
-            auth_data['identity']['methods'].append('password')
-            auth_data['identity']['password'] = self.build_password_auth(
-                user_id, username, user_domain_id, user_domain_name, password)
-        if kwargs:
-            auth_data['scope'] = self.build_auth_scope(**kwargs)
-        return {'auth': auth_data}
-
-
 class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
                       common_auth.AuthTestMixin):
 
-    def generate_token_schema(self, domain_scoped=False, project_scoped=False):
+    def generate_token_schema(self, system_scoped=False, domain_scoped=False,
+                              project_scoped=False):
         """Return a dictionary of token properties to validate against."""
+        ROLES_SCHEMA = {
+            'type': 'array',
+            'items': {
+                'type': 'object',
+                'properties': {
+                    'id': {'type': 'string', },
+                    'name': {'type': 'string', },
+                },
+                'required': ['id', 'name', ],
+                'additionalProperties': False,
+            },
+            'minItems': 1,
+        }
+
         properties = {
             'audit_ids': {
                 'type': 'array',
@@ -163,7 +95,7 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
             },
             'user': {
                 'type': 'object',
-                'required': ['id', 'name', 'domain'],
+                'required': ['id', 'name', 'domain', 'password_expires_at'],
                 'properties': {
                     'id': {'type': 'string'},
                     'name': {'type': 'string'},
@@ -175,42 +107,46 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
                         },
                         'required': ['id', 'name'],
                         'additonalProperties': False,
+                    },
+                    'password_expires_at': {
+                        'type': ['string', 'null'],
+                        'pattern': unit.TIME_FORMAT_REGEX,
                     }
                 },
                 'additionalProperties': False,
             }
         }
 
-        if domain_scoped:
+        if system_scoped:
             properties['catalog'] = {'type': 'array'}
-            properties['roles'] = {
-                'type': 'array',
-                'items': {
-                    'type': 'object',
-                    'properties': {
-                        'id': {'type': 'string', },
-                        'name': {'type': 'string', },
-                    },
-                    'required': ['id', 'name', ],
-                    'additionalProperties': False,
-                },
-                'minItems': 1,
-            }
-            properties['domain'] = {
-                'domain': {
-                    'type': 'object',
-                    'required': ['id', 'name'],
-                    'properties': {
-                        'id': {'type': 'string'},
-                        'name': {'type': 'string'}
-                    },
-                    'additionalProperties': False
+            properties['system'] = {
+                'type': 'object',
+                'properties': {
+                    'all': {'type': 'boolean'}
                 }
             }
+            properties['roles'] = ROLES_SCHEMA
+        elif domain_scoped:
+            properties['catalog'] = {'type': 'array'}
+            properties['roles'] = ROLES_SCHEMA
+            properties['domain'] = {
+                'type': 'object',
+                'required': ['id', 'name'],
+                'properties': {
+                    'id': {'type': 'string'},
+                    'name': {'type': 'string'}
+                },
+                'additionalProperties': False
+            }
         elif project_scoped:
             properties['is_admin_project'] = {'type': 'boolean'}
             properties['catalog'] = {'type': 'array'}
-            properties['roles'] = {'type': 'array'}
+            # FIXME(lbragstad): Remove this in favor of the predefined
+            # ROLES_SCHEMA dictionary once bug 1763510 is fixed.
+            ROLES_SCHEMA['items']['properties']['domain_id'] = {
+                'type': ['null', 'string', ],
+            }
+            properties['roles'] = ROLES_SCHEMA
             properties['is_domain'] = {'type': 'boolean'}
             properties['project'] = {
                 'type': ['object'],
@@ -240,12 +176,14 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
             'additionalProperties': False
         }
 
-        if domain_scoped:
+        if system_scoped:
+            schema['required'].extend(['system', 'roles'])
+            schema['optional'].append('catalog')
+        elif domain_scoped:
             schema['required'].extend(['domain', 'roles'])
             schema['optional'].append('catalog')
         elif project_scoped:
             schema['required'].append('project')
-            schema['optional'].append('bind')
             schema['optional'].append('catalog')
             schema['optional'].append('OS-TRUST:trust')
             schema['optional'].append('is_admin_project')
@@ -258,9 +196,11 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         config_files.append(unit.dirs.tests_conf('backend_sql.conf'))
         return config_files
 
-    def setUp(self, app_conf='keystone'):
+    def setUp(self, app_conf='keystone', enable_sqlite_foreign_key=False):
         """Setup for v3 Restful Test Cases."""
-        super(RestfulTestCase, self).setUp(app_conf=app_conf)
+        super(RestfulTestCase, self).setUp(
+            app_conf=app_conf,
+            enable_sqlite_foreign_key=enable_sqlite_foreign_key)
 
         self.empty_context = {'environment': {}}
 
@@ -270,31 +210,48 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
 
         super(RestfulTestCase, self).load_backends()
 
-    def load_fixtures(self, fixtures):
-        self.load_sample_data()
+    def load_fixtures(self, fixtures, enable_sqlite_foreign_key=False):
+        self.load_sample_data(
+            enable_sqlite_foreign_key=enable_sqlite_foreign_key)
 
-    def _populate_default_domain(self):
+    def _populate_default_domain(self, enable_sqlite_foreign_key=False):
         try:
-            self.resource_api.get_domain(DEFAULT_DOMAIN_ID)
+            PROVIDERS.resource_api.get_domain(DEFAULT_DOMAIN_ID)
         except exception.DomainNotFound:
+            # TODO(wxy): when FK is enabled in sqlite, a lot of tests will fail
+            # due to the root domain is missing. So we should open FKs for the
+            # tests one by one. If the FKs is enabled for one test,
+            # `enable_sqlite_foreign_key` should be `true` here to ensure the
+            # root domain is created. Once all tests enable FKs, the
+            # ``enable_sqlite_foreign_key`` can be removed.
+            if enable_sqlite_foreign_key:
+                root_domain = unit.new_domain_ref(
+                    id=resource_base.NULL_DOMAIN_ID,
+                    name=resource_base.NULL_DOMAIN_ID
+                )
+                self.resource_api.create_domain(resource_base.NULL_DOMAIN_ID,
+                                                root_domain)
             domain = unit.new_domain_ref(
                 description=(u'The default domain'),
                 id=DEFAULT_DOMAIN_ID,
                 name=u'Default')
-            self.resource_api.create_domain(DEFAULT_DOMAIN_ID, domain)
+            PROVIDERS.resource_api.create_domain(DEFAULT_DOMAIN_ID, domain)
 
-    def load_sample_data(self, create_region_and_endpoints=True):
-        self._populate_default_domain()
+    def load_sample_data(self, create_region_and_endpoints=True,
+                         enable_sqlite_foreign_key=False):
+        self._populate_default_domain(
+            enable_sqlite_foreign_key=enable_sqlite_foreign_key)
         self.domain = unit.new_domain_ref()
         self.domain_id = self.domain['id']
-        self.resource_api.create_domain(self.domain_id, self.domain)
+        PROVIDERS.resource_api.create_domain(self.domain_id, self.domain)
 
         self.project = unit.new_project_ref(domain_id=self.domain_id)
         self.project_id = self.project['id']
-        self.project = self.resource_api.create_project(self.project_id,
-                                                        self.project)
+        self.project = PROVIDERS.resource_api.create_project(
+            self.project_id, self.project
+        )
 
-        self.user = unit.create_user(self.identity_api,
+        self.user = unit.create_user(PROVIDERS.identity_api,
                                      domain_id=self.domain_id)
         self.user_id = self.user['id']
 
@@ -302,32 +259,33 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         self.default_domain_project = unit.new_project_ref(
             domain_id=DEFAULT_DOMAIN_ID)
         self.default_domain_project['id'] = self.default_domain_project_id
-        self.resource_api.create_project(self.default_domain_project_id,
-                                         self.default_domain_project)
+        PROVIDERS.resource_api.create_project(
+            self.default_domain_project_id, self.default_domain_project
+        )
 
         self.default_domain_user = unit.create_user(
-            self.identity_api,
+            PROVIDERS.identity_api,
             domain_id=DEFAULT_DOMAIN_ID)
         self.default_domain_user_id = self.default_domain_user['id']
 
         # create & grant policy.json's default role for admin_required
         self.role = unit.new_role_ref(name='admin')
         self.role_id = self.role['id']
-        self.role_api.create_role(self.role_id, self.role)
-        self.assignment_api.add_role_to_user_and_project(
+        PROVIDERS.role_api.create_role(self.role_id, self.role)
+        PROVIDERS.assignment_api.add_role_to_user_and_project(
             self.user_id, self.project_id, self.role_id)
-        self.assignment_api.add_role_to_user_and_project(
+        PROVIDERS.assignment_api.add_role_to_user_and_project(
             self.default_domain_user_id, self.default_domain_project_id,
             self.role_id)
-        self.assignment_api.add_role_to_user_and_project(
+        PROVIDERS.assignment_api.add_role_to_user_and_project(
             self.default_domain_user_id, self.project_id,
             self.role_id)
 
         # Create "req_admin" user for simulating a real user instead of the
         # admin_token_auth middleware
-        self.user_reqadmin = unit.create_user(self.identity_api,
+        self.user_reqadmin = unit.create_user(PROVIDERS.identity_api,
                                               DEFAULT_DOMAIN_ID)
-        self.assignment_api.add_role_to_user_and_project(
+        PROVIDERS.assignment_api.add_role_to_user_and_project(
             self.user_reqadmin['id'],
             self.default_domain_project_id,
             self.role_id)
@@ -335,19 +293,21 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         if create_region_and_endpoints:
             self.region = unit.new_region_ref()
             self.region_id = self.region['id']
-            self.catalog_api.create_region(self.region)
+            PROVIDERS.catalog_api.create_region(self.region)
 
             self.service = unit.new_service_ref()
             self.service_id = self.service['id']
-            self.catalog_api.create_service(self.service_id,
-                                            self.service.copy())
+            PROVIDERS.catalog_api.create_service(
+                self.service_id, self.service.copy()
+            )
 
             self.endpoint = unit.new_endpoint_ref(service_id=self.service_id,
                                                   interface='public',
                                                   region_id=self.region_id)
             self.endpoint_id = self.endpoint['id']
-            self.catalog_api.create_endpoint(self.endpoint_id,
-                                             self.endpoint.copy())
+            PROVIDERS.catalog_api.create_endpoint(
+                self.endpoint_id, self.endpoint.copy()
+            )
             # The server adds 'enabled' and defaults to True.
             self.endpoint['enabled'] = True
 
@@ -444,6 +404,32 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
             })
         return r.headers.get('X-Subject-Token')
 
+    def get_system_scoped_token(self):
+        """Convenience method for requesting system scoped tokens."""
+        r = self.admin_request(
+            method='POST',
+            path='/v3/auth/tokens',
+            body={
+                'auth': {
+                    'identity': {
+                        'methods': ['password'],
+                        'password': {
+                            'user': {
+                                'name': self.user['name'],
+                                'password': self.user['password'],
+                                'domain': {
+                                    'id': self.user['domain_id']
+                                }
+                            }
+                        }
+                    },
+                    'scope': {
+                        'system': {'all': True}
+                    }
+                }
+            })
+        return r.headers.get('X-Subject-Token')
+
     def get_domain_scoped_token(self):
         """Convenience method for requesting domain scoped token."""
         r = self.admin_request(
@@ -633,13 +619,16 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
 
     def assertValidISO8601ExtendedFormatDatetime(self, dt):
         try:
-            return timeutils.parse_strtime(dt, fmt=TIME_FORMAT)
+            return datetime.datetime.strptime(dt, TIME_FORMAT)
         except Exception:
             msg = '%s is not a valid ISO 8601 extended format date time.' % dt
             raise AssertionError(msg)
 
-    def assertValidTokenResponse(self, r, user=None):
-        self.assertTrue(r.headers.get('X-Subject-Token'))
+    def assertValidTokenResponse(self, r, user=None, forbid_token_id=False):
+        if forbid_token_id:
+            self.assertNotIn('X-Subject-Token', r.headers)
+        else:
+            self.assertTrue(r.headers.get('X-Subject-Token'))
         token = r.result['token']
 
         self.assertIsNotNone(token.get('expires_at'))
@@ -764,30 +753,19 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
 
         return token
 
-    def assertEqualTokens(self, a, b):
-        """Assert that two tokens are equal.
+    def assertValidSystemScopedTokenResponse(self, r, *args, **kwargs):
+        token = self.assertValidTokenResponse(r)
+        self.assertTrue(token['system']['all'])
 
-        Compare two tokens except for their ids. This also truncates
-        the time in the comparison.
-        """
-        def normalize(token):
-            del token['token']['expires_at']
-            del token['token']['issued_at']
-            return token
+        system_scoped_token_schema = self.generate_token_schema(
+            system_scoped=True
+        )
+        validator_object = validators.SchemaValidator(
+            system_scoped_token_schema
+        )
+        validator_object.validate(token)
 
-        a_expires_at = self.assertValidISO8601ExtendedFormatDatetime(
-            a['token']['expires_at'])
-        b_expires_at = self.assertValidISO8601ExtendedFormatDatetime(
-            b['token']['expires_at'])
-        self.assertCloseEnoughForGovernmentWork(a_expires_at, b_expires_at)
-
-        a_issued_at = self.assertValidISO8601ExtendedFormatDatetime(
-            a['token']['issued_at'])
-        b_issued_at = self.assertValidISO8601ExtendedFormatDatetime(
-            b['token']['issued_at'])
-        self.assertCloseEnoughForGovernmentWork(a_issued_at, b_issued_at)
-
-        return self.assertDictEqual(normalize(a), normalize(b))
+        return token
 
     # catalog validation
 
@@ -1099,7 +1077,7 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
                                               resource_url=None):
         entities = resp.result.get('role_assignments')
 
-        if expected_length:
+        if expected_length or expected_length == 0:
             self.assertEqual(expected_length, len(entities))
 
         # Collections should have relational links
@@ -1130,9 +1108,12 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         if entity['scope'].get('project'):
             self.assertIsNone(entity['scope'].get('domain'))
             self.assertIsNotNone(entity['scope']['project'].get('id'))
-        else:
+        elif entity['scope'].get('domain'):
             self.assertIsNotNone(entity['scope'].get('domain'))
             self.assertIsNotNone(entity['scope']['domain'].get('id'))
+        else:
+            self.assertIsNotNone(entity['scope'].get('system'))
+            self.assertTrue(entity['scope']['system']['all'])
 
         # An assignment link should be present
         self.assertIsNotNone(entity.get('links'))
@@ -1238,6 +1219,8 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
                 self.assertValidRole(role)
 
             self.assertValidListLinks(entity.get('roles_links'))
+            # Mirror the test tempest does to ensure the self-link is correct
+            self.assertIn('v3/OS-TRUST/trusts', entity.get('links')['self'])
 
             # always disallow role xor project_id (neither or both is allowed)
             has_roles = bool(entity.get('roles'))
@@ -1269,88 +1252,30 @@ class RestfulTestCase(unit.SQLDriverOverrides, rest.RestfulTestCase,
         for attribute in attributes:
             self.assertIsNotNone(entity.get(attribute))
 
-    def assertValidServiceProviderListResponse(self, resp, *args, **kwargs):
-        if kwargs.get('keys_to_check') is None:
-            kwargs['keys_to_check'] = ['auth_url', 'id', 'enabled',
-                                       'description', 'relay_state_prefix',
-                                       'sp_url']
-        return self.assertValidListResponse(
-            resp,
-            'service_providers',
-            self.assertValidServiceProvider,
-            *args,
-            **kwargs)
-
     def build_external_auth_request(self, remote_user,
                                     remote_domain=None, auth_data=None,
                                     kerberos=False):
-        environment = {'REMOTE_USER': remote_user, 'AUTH_TYPE': 'Negotiate'}
-        if remote_domain:
-            environment['REMOTE_DOMAIN'] = remote_domain
+        environment = self.build_external_auth_environ(
+            remote_user, remote_domain)
         if not auth_data:
             auth_data = self.build_authentication_request(
                 kerberos=kerberos)['auth']
-        auth_info = auth.controllers.AuthInfo.create(auth_data)
-        auth_context = {'extras': {}, 'method_names': []}
+        auth_info = auth.core.AuthInfo.create(auth_data)
+        auth_context = auth.core.AuthContext(method_names=[])
         return self.make_request(environ=environment), auth_info, auth_context
 
+    def build_external_auth_environ(self, remote_user, remote_domain=None):
+        environment = {'REMOTE_USER': remote_user, 'AUTH_TYPE': 'Negotiate'}
+        if remote_domain:
+            environment['REMOTE_DOMAIN'] = remote_domain
+        return environment
+
 
 class VersionTestCase(RestfulTestCase):
     def test_get_version(self):
         pass
 
 
-# NOTE(morganfainberg): To be removed when admin_token_auth is removed. This
-# has been split out to allow testing admin_token auth without enabling it
-# for other tests.
-class AuthContextMiddlewareAdminTokenTestCase(RestfulTestCase):
-    EXTENSION_TO_ADD = 'admin_token_auth'
-
-    def config_overrides(self):
-        super(AuthContextMiddlewareAdminTokenTestCase, self).config_overrides()
-        self.config_fixture.config(
-            admin_token='ADMIN')
-
-    # NOTE(morganfainberg): This is knowingly copied from below for simplicity
-    # during the deprecation cycle.
-    def _middleware_request(self, token, extra_environ=None):
-
-        def application(environ, start_response):
-            body = b'body'
-            headers = [('Content-Type', 'text/html; charset=utf8'),
-                       ('Content-Length', str(len(body)))]
-            start_response('200 OK', headers)
-            return [body]
-
-        app = webtest.TestApp(middleware.AuthContextMiddleware(application),
-                              extra_environ=extra_environ)
-        resp = app.get('/', headers={middleware.AUTH_TOKEN_HEADER: token})
-        self.assertEqual('body', resp.text)  # just to make sure it worked
-        return resp.request
-
-    def test_admin_auth_context(self):
-        # test to make sure AuthContextMiddleware does not attempt to build the
-        # auth context if the admin_token middleware indicates it's admin
-        # already.
-        token_id = uuid.uuid4().hex  # token doesn't matter.
-        # the admin_token middleware sets is_admin in the context.
-        extra_environ = {middleware.CONTEXT_ENV: {'is_admin': True}}
-        req = self._middleware_request(token_id, extra_environ)
-        auth_context = req.environ.get(authorization.AUTH_CONTEXT_ENV)
-        self.assertDictEqual({}, auth_context)
-
-    @mock.patch.object(middleware_auth.versionutils,
-                       'report_deprecated_feature')
-    def test_admin_token_auth_context_deprecated(self, mock_report_deprecated):
-        # For backwards compatibility AuthContextMiddleware will check that the
-        # admin token (as configured in the CONF file) is present and not
-        # attempt to build the auth context. This is deprecated.
-        req = self._middleware_request('ADMIN')
-        auth_context = req.environ.get(authorization.AUTH_CONTEXT_ENV)
-        self.assertDictEqual({}, auth_context)
-        self.assertEqual(1, mock_report_deprecated.call_count)
-
-
 # NOTE(gyee): test AuthContextMiddleware here instead of test_middleware.py
 # because we need the token
 class AuthContextMiddlewareTestCase(RestfulTestCase):
@@ -1366,7 +1291,7 @@ class AuthContextMiddlewareTestCase(RestfulTestCase):
 
         app = webtest.TestApp(middleware.AuthContextMiddleware(application),
                               extra_environ=extra_environ)
-        resp = app.get('/', headers={middleware.AUTH_TOKEN_HEADER: token})
+        resp = app.get('/', headers={authorization.AUTH_TOKEN_HEADER: token})
         self.assertEqual(b'body', resp.body)  # just to make sure it worked
         return resp.request
 
@@ -1393,10 +1318,14 @@ class AuthContextMiddlewareTestCase(RestfulTestCase):
     def test_unscoped_token_auth_context(self):
         unscoped_token = self.get_unscoped_token()
         req = self._middleware_request(unscoped_token)
+        # This check originally looked that the value was unset
+        # but that was an artifact of the custom context keystone
+        # used to create.  Oslo-context will always provide the
+        # same set of keys, but the values will be None in an
+        # unscoped token
         for key in ['project_id', 'domain_id', 'domain_name']:
-            self.assertNotIn(
-                key,
-                req.environ.get(authorization.AUTH_CONTEXT_ENV))
+            self.assertIsNone(
+                req.environ.get(authorization.AUTH_CONTEXT_ENV)[key])
 
     def test_project_scoped_token_auth_context(self):
         project_scoped_token = self.get_scoped_token()
@@ -1437,11 +1366,12 @@ class AuthContextMiddlewareTestCase(RestfulTestCase):
         req_context = oslo_context.context.get_current()
         self.assertEqual(request_id, req_context.request_id)
         self.assertEqual(token, req_context.auth_token)
-        self.assertEqual(self.user['id'], req_context.user)
-        self.assertEqual(self.project['id'], req_context.tenant)
-        self.assertIsNone(req_context.domain)
-        self.assertEqual(self.user['domain_id'], req_context.user_domain)
-        self.assertEqual(self.project['domain_id'], req_context.project_domain)
+        self.assertEqual(self.user['id'], req_context.user_id)
+        self.assertEqual(self.project['id'], req_context.project_id)
+        self.assertIsNone(req_context.domain_id)
+        self.assertEqual(self.user['domain_id'], req_context.user_domain_id)
+        self.assertEqual(self.project['domain_id'],
+                         req_context.project_domain_id)
         self.assertFalse(req_context.is_admin)
 
 
@@ -1505,6 +1435,8 @@ class AssignmentTestMixin(object):
         """
         if attribs.get('domain_id'):
             link = '/domains/' + attribs['domain_id']
+        elif attribs.get('system'):
+            link = '/system'
         else:
             link = '/projects/' + attribs['project_id']
 
@@ -1532,6 +1464,8 @@ class AssignmentTestMixin(object):
 
         if attribs.get('domain_id'):
             entity['scope'] = {'domain': {'id': attribs['domain_id']}}
+        elif attribs.get('system'):
+            entity['scope'] = {'system': {'all': True}}
         else:
             entity['scope'] = {'project': {'id': attribs['project_id']}}
 
@@ -1570,7 +1504,7 @@ class AssignmentTestMixin(object):
         entity = {'links': {}}
         attributes_for_links = {}
         if project_ref:
-            dmn_name = self.resource_api.get_domain(
+            dmn_name = PROVIDERS.resource_api.get_domain(
                 project_ref['domain_id'])['name']
 
             entity['scope'] = {'project': {
@@ -1585,7 +1519,7 @@ class AssignmentTestMixin(object):
                                           'name': domain_ref['name']}}
             attributes_for_links['domain_id'] = domain_ref['id']
         if user_ref:
-            dmn_name = self.resource_api.get_domain(
+            dmn_name = PROVIDERS.resource_api.get_domain(
                 user_ref['domain_id'])['name']
             entity['user'] = {'id': user_ref['id'],
                               'name': user_ref['name'],
@@ -1593,7 +1527,7 @@ class AssignmentTestMixin(object):
                                          'name': dmn_name}}
             attributes_for_links['user_id'] = user_ref['id']
         else:
-            dmn_name = self.resource_api.get_domain(
+            dmn_name = PROVIDERS.resource_api.get_domain(
                 group_ref['domain_id'])['name']
             entity['group'] = {'id': group_ref['id'],
                                'name': group_ref['name'],
@@ -1605,6 +1539,11 @@ class AssignmentTestMixin(object):
         if role_ref:
             entity['role'] = {'id': role_ref['id'],
                               'name': role_ref['name']}
+            if role_ref['domain_id']:
+                dmn_name = PROVIDERS.resource_api.get_domain(
+                    role_ref['domain_id'])['name']
+                entity['role']['domain'] = {'id': role_ref['domain_id'],
+                                            'name': dmn_name}
             attributes_for_links['role_id'] = role_ref['id']
 
         if inherited_assignment:
