@@ -17,20 +17,24 @@ import json
 import uuid
 
 from keystoneclient.contrib.ec2 import utils as ec2_utils
+import mock
+from oslo_db import exception as oslo_db_exception
+from oslo_serialization import jsonutils
 from six.moves import http_client
 from testtools import matchers
 
+from keystone.common import provider_api
 from keystone.common import utils
-import keystone.conf
 from keystone.contrib.ec2 import controllers
 from keystone.credential.providers import fernet as credential_fernet
 from keystone import exception
 from keystone.tests import unit
 from keystone.tests.unit import ksfixtures
+from keystone.tests.unit.ksfixtures import temporaryfile
 from keystone.tests.unit import test_v3
 
 
-CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 CRED_TYPE_EC2 = controllers.CRED_TYPE_EC2
 
 
@@ -57,7 +61,7 @@ class CredentialBaseTestCase(test_v3.RestfulTestCase):
         credential_id = credential['id']
 
         # Create direct via the DB API to avoid validation failure
-        self.credential_api.create_credential(credential_id, credential)
+        PROVIDERS.credential_api.create_credential(credential_id, credential)
 
         return json.dumps(blob), credential_id
 
@@ -72,24 +76,26 @@ class CredentialTestCase(CredentialBaseTestCase):
         self.credential = unit.new_credential_ref(user_id=self.user['id'],
                                                   project_id=self.project_id)
 
-        self.credential_api.create_credential(
+        PROVIDERS.credential_api.create_credential(
             self.credential['id'],
             self.credential)
 
     def test_credential_api_delete_credentials_for_project(self):
-        self.credential_api.delete_credentials_for_project(self.project_id)
+        PROVIDERS.credential_api.delete_credentials_for_project(
+            self.project_id
+        )
         # Test that the credential that we created in .setUp no longer exists
         # once we delete all credentials for self.project_id
         self.assertRaises(exception.CredentialNotFound,
-                          self.credential_api.get_credential,
+                          PROVIDERS.credential_api.get_credential,
                           credential_id=self.credential['id'])
 
     def test_credential_api_delete_credentials_for_user(self):
-        self.credential_api.delete_credentials_for_user(self.user_id)
+        PROVIDERS.credential_api.delete_credentials_for_user(self.user_id)
         # Test that the credential that we created in .setUp no longer exists
         # once we delete all credentials for self.user_id
         self.assertRaises(exception.CredentialNotFound,
-                          self.credential_api.get_credential,
+                          PROVIDERS.credential_api.get_credential,
                           credential_id=self.credential['id'])
 
     def test_list_credentials(self):
@@ -100,7 +106,9 @@ class CredentialTestCase(CredentialBaseTestCase):
     def test_list_credentials_filtered_by_user_id(self):
         """Call ``GET  /credentials?user_id={user_id}``."""
         credential = unit.new_credential_ref(user_id=uuid.uuid4().hex)
-        self.credential_api.create_credential(credential['id'], credential)
+        PROVIDERS.credential_api.create_credential(
+            credential['id'], credential
+        )
 
         r = self.get('/credentials?user_id=%s' % self.user['id'])
         self.assertValidCredentialListResponse(r, ref=self.credential)
@@ -115,7 +123,7 @@ class CredentialTestCase(CredentialBaseTestCase):
                                                  project_id=self.project_id,
                                                  type=CRED_TYPE_EC2)
 
-        ec2_resp = self.credential_api.create_credential(
+        ec2_resp = PROVIDERS.credential_api.create_credential(
             ec2_credential['id'], ec2_credential)
 
         # The type cert was chosen for the same reason as ec2
@@ -145,11 +153,11 @@ class CredentialTestCase(CredentialBaseTestCase):
         credential_user1_cert = unit.new_credential_ref(user_id=user1_id)
         credential_user2_cert = unit.new_credential_ref(user_id=user2_id)
 
-        self.credential_api.create_credential(
+        PROVIDERS.credential_api.create_credential(
             credential_user1_ec2['id'], credential_user1_ec2)
-        self.credential_api.create_credential(
+        PROVIDERS.credential_api.create_credential(
             credential_user1_cert['id'], credential_user1_cert)
-        self.credential_api.create_credential(
+        PROVIDERS.credential_api.create_credential(
             credential_user2_cert['id'], credential_user2_cert)
 
         r = self.get('/credentials?user_id=%s&type=ec2' % user1_id)
@@ -246,6 +254,38 @@ class CredentialTestCase(CredentialBaseTestCase):
             '/credentials/%(credential_id)s' % {
                 'credential_id': self.credential['id']})
 
+    def test_delete_credential_retries_on_deadlock(self):
+        patcher = mock.patch('sqlalchemy.orm.query.Query.delete',
+                             autospec=True)
+
+        class FakeDeadlock(object):
+            def __init__(self, mock_patcher):
+                self.deadlock_count = 2
+                self.mock_patcher = mock_patcher
+                self.patched = True
+
+            def __call__(self, *args, **kwargs):
+                if self.deadlock_count > 1:
+                    self.deadlock_count -= 1
+                else:
+                    self.mock_patcher.stop()
+                    self.patched = False
+                raise oslo_db_exception.DBDeadlock
+
+        sql_delete_mock = patcher.start()
+        side_effect = FakeDeadlock(patcher)
+        sql_delete_mock.side_effect = side_effect
+
+        try:
+            PROVIDERS.credential_api.delete_credentials_for_user(
+                user_id=self.user['id'])
+        finally:
+            if side_effect.patched:
+                patcher.stop()
+
+        # initial attempt + 1 retry
+        self.assertEqual(sql_delete_mock.call_count, 2)
+
     def test_create_ec2_credential(self):
         """Call ``POST /credentials`` for creating ec2 credential."""
         blob, ref = unit.new_ec2_credential(user_id=self.user['id'],
@@ -343,6 +383,95 @@ class CredentialTestCase(CredentialBaseTestCase):
         self.assertValidCredentialResponse(r, ref)
 
 
+class CredentialSelfServiceTestCase(CredentialBaseTestCase):
+    """Test self-service credential CRUD."""
+
+    def _policy_fixture(self):
+        return ksfixtures.Policy(self.tmpfilename, self.config_fixture)
+
+    def _set_policy(self, new_policy):
+        with open(self.tmpfilename, "w") as policyfile:
+            policyfile.write(jsonutils.dumps(new_policy))
+
+    def setUp(self):
+        self.tempfile = self.useFixture(temporaryfile.SecureTempFile())
+        self.tmpfilename = self.tempfile.file_name
+        super(CredentialSelfServiceTestCase, self).setUp()
+
+        # set the self-service credential policies
+        self_service_credential_policies = {
+            "identity:create_credential": "user_id:%(credential.user_id)s",
+            "identity:list_credentials": "user_id:%(user_id)s",
+            "identity:get_credential": "user_id:%(target.credential.user_id)s",
+            "identity:update_credential":
+                "user_id:%(target.credential.user_id)s",
+            "identity:delete_credential":
+                "user_id:%(target.credential.user_id)s"
+        }
+        self._set_policy(self_service_credential_policies)
+
+        # remove the 'admin' role from user and replace it with an
+        # arbitrary role
+        PROVIDERS.assignment_api.remove_role_from_user_and_project(
+            self.user_id, self.project_id, self.role_id)
+        self.arbitrary_role = unit.new_role_ref(name=uuid.uuid4().hex)
+        PROVIDERS.role_api.create_role(self.arbitrary_role['id'],
+                                       self.arbitrary_role)
+        PROVIDERS.assignment_api.add_role_to_user_and_project(
+            self.user_id, self.project_id, self.arbitrary_role['id'])
+
+        self.credential = unit.new_credential_ref(user_id=self.user['id'],
+                                                  project_id=self.project_id)
+
+        PROVIDERS.credential_api.create_credential(
+            self.credential['id'],
+            self.credential)
+
+    def test_list_credentials_filtered_by_user_id(self):
+        """Call ``GET  /credentials?user_id={user_id}``."""
+        credential = unit.new_credential_ref(user_id=uuid.uuid4().hex)
+        PROVIDERS.credential_api.create_credential(
+            credential['id'], credential
+        )
+
+        r = self.get('/credentials?user_id=%s' % self.user['id'])
+        self.assertValidCredentialListResponse(r, ref=self.credential)
+        for cred in r.result['credentials']:
+            self.assertEqual(self.user['id'], cred['user_id'])
+
+    def test_create_credential(self):
+        """Call ``POST /credentials``."""
+        ref = unit.new_credential_ref(user_id=self.user['id'])
+        r = self.post(
+            '/credentials',
+            body={'credential': ref})
+        self.assertValidCredentialResponse(r, ref)
+
+    def test_get_credential(self):
+        """Call ``GET /credentials/{credential_id}``."""
+        r = self.get(
+            '/credentials/%(credential_id)s' % {
+                'credential_id': self.credential['id']})
+        self.assertValidCredentialResponse(r, self.credential)
+
+    def test_update_credential(self):
+        """Call ``PATCH /credentials/{credential_id}``."""
+        ref = unit.new_credential_ref(user_id=self.user['id'],
+                                      project_id=self.project_id)
+        del ref['id']
+        r = self.patch(
+            '/credentials/%(credential_id)s' % {
+                'credential_id': self.credential['id']},
+            body={'credential': ref})
+        self.assertValidCredentialResponse(r, ref)
+
+    def test_delete_credential(self):
+        """Call ``DELETE /credentials/{credential_id}``."""
+        self.delete(
+            '/credentials/%(credential_id)s' % {
+                'credential_id': self.credential['id']})
+
+
 class TestCredentialTrustScoped(test_v3.RestfulTestCase):
     """Test credential with trust scoped token."""
 
@@ -351,7 +480,9 @@ class TestCredentialTrustScoped(test_v3.RestfulTestCase):
 
         self.trustee_user = unit.new_user_ref(domain_id=self.domain_id)
         password = self.trustee_user['password']
-        self.trustee_user = self.identity_api.create_user(self.trustee_user)
+        self.trustee_user = PROVIDERS.identity_api.create_user(
+            self.trustee_user
+        )
         self.trustee_user['password'] = password
         self.trustee_user_id = self.trustee_user['id']
         self.useFixture(
@@ -364,7 +495,7 @@ class TestCredentialTrustScoped(test_v3.RestfulTestCase):
 
     def config_overrides(self):
         super(TestCredentialTrustScoped, self).config_overrides()
-        self.config_fixture.config(group='trust', enabled=True)
+        self.config_fixture.config(group='trust')
 
     def test_trust_scoped_ec2_credential(self):
         """Test creating trust scoped ec2 credential.
@@ -423,9 +554,6 @@ class TestCredentialTrustScoped(test_v3.RestfulTestCase):
 class TestCredentialEc2(CredentialBaseTestCase):
     """Test v3 credential compatibility with ec2tokens."""
 
-    def setUp(self):
-        super(TestCredentialEc2, self).setUp()
-
     def _validate_signature(self, access, secret):
         """Test signature validation with the access/secret provided."""
         signer = ec2_utils.Ec2Signer(secret)
@@ -510,7 +638,7 @@ class TestCredentialEc2(CredentialBaseTestCase):
             user_id=self.user_id,
             project_id=self.project_id)
         non_ec2_cred['id'] = cred_id
-        self.credential_api.create_credential(cred_id, non_ec2_cred)
+        PROVIDERS.credential_api.create_credential(cred_id, non_ec2_cred)
         uri = '/'.join([self._get_ec2_cred_uri(), access_key])
         # if access_key is not found, ec2 controller raises Unauthorized
         # exception
@@ -531,8 +659,9 @@ class TestCredentialEc2(CredentialBaseTestCase):
             user_id=self.user_id,
             project_id=self.project_id)
         non_ec2_cred['type'] = uuid.uuid4().hex
-        self.credential_api.create_credential(non_ec2_cred['id'],
-                                              non_ec2_cred)
+        PROVIDERS.credential_api.create_credential(
+            non_ec2_cred['id'], non_ec2_cred
+        )
         r = self.get(uri)
         cred_list_2 = r.result['credentials']
         # still one element because non-EC2 credentials are not returned.
@@ -544,10 +673,10 @@ class TestCredentialEc2(CredentialBaseTestCase):
         ec2_cred = self._get_ec2_cred()
         uri = '/'.join([self._get_ec2_cred_uri(), ec2_cred['access']])
         cred_from_credential_api = (
-            self.credential_api
+            PROVIDERS.credential_api
             .list_credentials_for_user(self.user_id, type=CRED_TYPE_EC2))
         self.assertEqual(1, len(cred_from_credential_api))
         self.delete(uri)
         self.assertRaises(exception.CredentialNotFound,
-                          self.credential_api.get_credential,
+                          PROVIDERS.credential_api.get_credential,
                           cred_from_credential_api[0]['id'])
